@@ -12,7 +12,9 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#ifdef ENABLE_DRM
+#include "virgl_util.h"
+
+#ifdef ENABLE_DRM_MSM
 
 int drm_renderer_init(int drm_fd);
 
@@ -29,7 +31,7 @@ struct virgl_context *drm_renderer_create(size_t debug_len, const char *debug_na
 static inline int
 drm_renderer_init(UNUSED int drm_fd)
 {
-   virgl_log("DRM native context support was not enabled in virglrenderer\n");
+   virgl_error("DRM native context support was not enabled in virglrenderer\n");
    return -1;
 }
 
