@@ -25,6 +25,8 @@
  * 
  **************************************************************************/
 
+#include <inttypes.h>
+
 #include "util/u_debug.h"
 #include "util/u_string.h"
 #include "util/u_math.h"
@@ -51,24 +53,34 @@ struct dump_ctx
    int indent;
    
    uint indentation;
-   FILE *file;
+   tgsi_dump_callback_type logger;
+   void *user_data;
 
    void (*dump_printf)(struct dump_ctx *ctx, const char *format, ...);
 };
 
-static void 
+static void
 dump_ctx_printf(struct dump_ctx *ctx, const char *format, ...)
 {
    va_list ap;
-   (void)ctx;
    va_start(ap, format);
-   if (ctx->file)
-      vfprintf(ctx->file, format, ap);
+   if (ctx->logger)
+      ctx->logger(format, ap, ctx->user_data);
    else
       _debug_vprintf(format, ap);
    va_end(ap);
 }
 
+static void
+log_to_file(const char *fmt, va_list ap, void* user_data)
+{
+   FILE *file = user_data;
+   if (file)
+      vfprintf(file, fmt, ap);
+   else
+      _debug_vprintf(fmt, ap);
+}
+
 static void
 dump_enum(
    struct dump_ctx *ctx,
@@ -87,6 +99,8 @@ dump_enum(
 #define CHR(C)          ctx->dump_printf( ctx, "%c", C )
 #define UIX(I)          ctx->dump_printf( ctx, "0x%x", I )
 #define UID(I)          ctx->dump_printf( ctx, "%u", I )
+#define SI64D(I)        ctx->dump_printf( ctx, "%"PRId64, I )
+#define UI64D(I)        ctx->dump_printf( ctx, "%"PRIu64, I )
 #define INSTID(I)       ctx->dump_printf( ctx, "% 3u", I )
 #define SID(I)          ctx->dump_printf( ctx, "%d", I )
 #define FLT(F)          ctx->dump_printf( ctx, "%10.4f", F )
@@ -233,7 +247,7 @@ _dump_writemask(
    }
 }
 
-static void
+static bool
 dump_imm_data(struct tgsi_iterate_context *iter,
               union tgsi_immediate_data *data,
               unsigned num_tokens,
@@ -244,7 +258,8 @@ dump_imm_data(struct tgsi_iterate_context *iter,
 
    TXT( " {" );
 
-   assert( num_tokens <= 4 );
+   if (num_tokens > 4 )
+       return false;
    for (i = 0; i < num_tokens; i++) {
       switch (data_type) {
       case TGSI_IMM_FLOAT64: {
@@ -254,6 +269,20 @@ dump_imm_data(struct tgsi_iterate_context *iter,
          i++;
          break;
       }
+      case TGSI_IMM_INT64: {
+         union di d;
+         d.i = data[i].Uint | (uint64_t)data[i+1].Uint << 32;
+         SI64D( d.i );
+         i++;
+         break;
+      }
+      case TGSI_IMM_UINT64: {
+         union di d;
+         d.ui = data[i].Uint | (uint64_t)data[i+1].Uint << 32;
+         UI64D( d.ui );
+         i++;
+         break;
+      }
       case TGSI_IMM_FLOAT32:
          if (ctx->dump_float_as_hex)
             HFLT( data[i].Float );
@@ -267,13 +296,14 @@ dump_imm_data(struct tgsi_iterate_context *iter,
          SID(data[i].Int);
          break;
       default:
-         assert( 0 );
+         return false;
       }
 
       if (i < num_tokens - 1)
          TXT( ", " );
    }
    TXT( "}" );
+   return true;
 }
 
 static boolean
@@ -348,15 +378,32 @@ iter_declaration(
       }
    }
 
-   if (decl->Declaration.File == TGSI_FILE_RESOURCE) {
+   if (decl->Declaration.File == TGSI_FILE_IMAGE) {
       TXT(", ");
-      ENM(decl->Resource.Resource, tgsi_texture_names);
-      if (decl->Resource.Writable)
+      ENM(decl->Image.Resource, tgsi_texture_names);
+      TXT(", ");
+      TXT(util_format_name(decl->Image.Format));
+      if (decl->Image.Writable)
          TXT(", WR");
-      if (decl->Resource.Raw)
+      if (decl->Image.Raw)
          TXT(", RAW");
    }
 
+   if (decl->Declaration.File == TGSI_FILE_BUFFER) {
+      if (decl->Declaration.Atomic)
+         TXT(", ATOMIC");
+   }
+
+   if (decl->Declaration.File == TGSI_FILE_MEMORY) {
+      switch (decl->Declaration.MemType) {
+      /* Note: ,GLOBAL is optional / the default */
+      case TGSI_MEMORY_TYPE_GLOBAL:  TXT(", GLOBAL");  break;
+      case TGSI_MEMORY_TYPE_SHARED:  TXT(", SHARED");  break;
+      case TGSI_MEMORY_TYPE_PRIVATE: TXT(", PRIVATE"); break;
+      case TGSI_MEMORY_TYPE_INPUT:   TXT(", INPUT");   break;
+      }
+   }
+
    if (decl->Declaration.File == TGSI_FILE_SAMPLER_VIEW) {
       TXT(", ");
       ENM(decl->SamplerView.Resource, tgsi_texture_names);
@@ -422,6 +469,8 @@ tgsi_dump_declaration(
    struct dump_ctx ctx;
 
    ctx.dump_printf = dump_ctx_printf;
+   ctx.logger = NULL;
+   ctx.user_data = NULL;
 
    iter_declaration( &ctx.iter, (struct tgsi_full_declaration *)decl );
 }
@@ -431,7 +480,7 @@ iter_property(
    struct tgsi_iterate_context *iter,
    struct tgsi_full_property *prop )
 {
-   unsigned i;
+   int i;
    struct dump_ctx *ctx = (struct dump_ctx *)iter;
 
    TXT( "PROPERTY " );
@@ -470,6 +519,8 @@ void tgsi_dump_property(
    struct dump_ctx ctx;
 
    ctx.dump_printf = dump_ctx_printf;
+   ctx.logger = NULL;
+   ctx.user_data = NULL;
 
    iter_property( &ctx.iter, (struct tgsi_full_property *)prop );
 }
@@ -486,8 +537,9 @@ iter_immediate(
    TXT( "] " );
    ENM( imm->Immediate.DataType, tgsi_immediate_type_names );
 
-   dump_imm_data(iter, imm->u, imm->Immediate.NrTokens - 1,
-                 imm->Immediate.DataType);
+   if (!dump_imm_data(iter, imm->u, imm->Immediate.NrTokens - 1,
+                      imm->Immediate.DataType))
+       return false;
 
    EOL();
 
@@ -501,6 +553,8 @@ tgsi_dump_immediate(
    struct dump_ctx ctx;
 
    ctx.dump_printf = dump_ctx_printf;
+   ctx.logger = NULL;
+   ctx.user_data = NULL;
 
    iter_immediate( &ctx.iter, (struct tgsi_full_immediate *)imm );
 }
@@ -524,30 +578,6 @@ iter_instruction(
       TXT( "  " );
    ctx->indent += info->post_indent;
 
-   if (inst->Instruction.Predicate) {
-      CHR( '(' );
-
-      if (inst->Predicate.Negate)
-         CHR( '!' );
-
-      TXT( "PRED[" );
-      SID( inst->Predicate.Index );
-      CHR( ']' );
-
-      if (inst->Predicate.SwizzleX != TGSI_SWIZZLE_X ||
-          inst->Predicate.SwizzleY != TGSI_SWIZZLE_Y ||
-          inst->Predicate.SwizzleZ != TGSI_SWIZZLE_Z ||
-          inst->Predicate.SwizzleW != TGSI_SWIZZLE_W) {
-         CHR( '.' );
-         ENM( inst->Predicate.SwizzleX, tgsi_swizzle_names );
-         ENM( inst->Predicate.SwizzleY, tgsi_swizzle_names );
-         ENM( inst->Predicate.SwizzleZ, tgsi_swizzle_names );
-         ENM( inst->Predicate.SwizzleW, tgsi_swizzle_names );
-      }
-
-      TXT( ") " );
-   }
-
    TXT( info->mnemonic );
 
    if (inst->Instruction.Saturate) {
@@ -617,17 +647,37 @@ iter_instruction(
       }
    }
 
-   switch (inst->Instruction.Opcode) {
-   case TGSI_OPCODE_IF:
-   case TGSI_OPCODE_UIF:
-   case TGSI_OPCODE_ELSE:
-   case TGSI_OPCODE_BGNLOOP:
-   case TGSI_OPCODE_ENDLOOP:
-   case TGSI_OPCODE_CAL:
-   case TGSI_OPCODE_BGNSUB:
-      TXT( " :" );
-      UID( inst->Label.Label );
-      break;
+   if (inst->Instruction.Memory) {
+      uint32_t qualifier = inst->Memory.Qualifier;
+      while (qualifier) {
+         int bit = ffs(qualifier) - 1;
+         qualifier &= ~(1U << bit);
+         TXT(", ");
+         ENM(bit, tgsi_memory_names);
+      }
+      if (inst->Memory.Texture) {
+         TXT( ", " );
+         ENM( inst->Memory.Texture, tgsi_texture_names );
+      }
+      if (inst->Memory.Format) {
+         TXT( ", " );
+         TXT( util_format_name(inst->Memory.Format) );
+      }
+   }
+
+   if (inst->Instruction.Label) {
+      switch (inst->Instruction.Opcode) {
+      case TGSI_OPCODE_IF:
+      case TGSI_OPCODE_UIF:
+      case TGSI_OPCODE_ELSE:
+      case TGSI_OPCODE_BGNLOOP:
+      case TGSI_OPCODE_ENDLOOP:
+      case TGSI_OPCODE_CAL:
+      case TGSI_OPCODE_BGNSUB:
+         TXT( " :" );
+         UID( inst->Label.Label );
+         break;
+      }
    }
 
    /* update indentation */
@@ -655,7 +705,8 @@ tgsi_dump_instruction(
    ctx.indent = 0;
    ctx.dump_printf = dump_ctx_printf;
    ctx.indentation = 0;
-   ctx.file = NULL;
+   ctx.logger = NULL;
+   ctx.user_data = NULL;
 
    iter_instruction( &ctx.iter, (struct tgsi_full_instruction *)inst );
 }
@@ -670,6 +721,39 @@ prolog(
    return TRUE;
 }
 
+
+void
+tgsi_dump_with_logger(
+   const struct tgsi_token *tokens,
+   uint flags,
+   tgsi_dump_callback_type logger,
+   void *user_data)
+{
+   struct dump_ctx ctx;
+
+   ctx.iter.prolog = prolog;
+   ctx.iter.iterate_instruction = iter_instruction;
+   ctx.iter.iterate_declaration = iter_declaration;
+   ctx.iter.iterate_immediate = iter_immediate;
+   ctx.iter.iterate_property = iter_property;
+   ctx.iter.epilog = NULL;
+
+   ctx.instno = 0;
+   ctx.immno = 0;
+   ctx.indent = 0;
+   ctx.dump_printf = dump_ctx_printf;
+   ctx.indentation = 0;
+   ctx.logger = logger;
+   ctx.user_data = user_data;
+
+   if (flags & TGSI_DUMP_FLOAT_AS_HEX)
+      ctx.dump_float_as_hex = TRUE;
+   else
+      ctx.dump_float_as_hex = FALSE;
+
+   tgsi_iterate_shader( tokens, &ctx.iter );
+}
+
 void
 tgsi_dump_to_file(const struct tgsi_token *tokens, uint flags, FILE *file)
 {
@@ -687,7 +771,8 @@ tgsi_dump_to_file(const struct tgsi_token *tokens, uint flags, FILE *file)
    ctx.indent = 0;
    ctx.dump_printf = dump_ctx_printf;
    ctx.indentation = 0;
-   ctx.file = file;
+   ctx.logger = log_to_file;
+   ctx.user_data = (void*)file;
 
    if (flags & TGSI_DUMP_FLOAT_AS_HEX)
       ctx.dump_float_as_hex = TRUE;
@@ -721,7 +806,7 @@ str_dump_ctx_printf(struct dump_ctx *ctx, const char *format, ...)
       int written;
       va_list ap;
       va_start(ap, format);
-      written = util_vsnprintf(sctx->ptr, sctx->left, format, ap);
+      written = vsnprintf(sctx->ptr, sctx->left, format, ap);
       va_end(ap);
 
       /* Some complicated logic needed to handle the return value of
@@ -757,7 +842,8 @@ tgsi_dump_str(
    ctx.base.indent = 0;
    ctx.base.dump_printf = &str_dump_ctx_printf;
    ctx.base.indentation = 0;
-   ctx.base.file = NULL;
+   ctx.base.logger = NULL;
+   ctx.base.user_data = NULL;
 
    ctx.str = str;
    ctx.str[0] = 0;
@@ -770,9 +856,9 @@ tgsi_dump_str(
    else
       ctx.base.dump_float_as_hex = FALSE;
 
-   tgsi_iterate_shader( tokens, &ctx.base.iter );
+   boolean success = tgsi_iterate_shader( tokens, &ctx.base.iter );
 
-   return !ctx.nospace;
+   return !ctx.nospace && success;
 }
 
 void
@@ -789,7 +875,8 @@ tgsi_dump_instruction_str(
    ctx.base.indent = 0;
    ctx.base.dump_printf = &str_dump_ctx_printf;
    ctx.base.indentation = 0;
-   ctx.base.file = NULL;
+   ctx.base.logger = NULL;
+   ctx.base.user_data = NULL;
 
    ctx.str = str;
    ctx.str[0] = 0;
