@@ -257,7 +257,6 @@ util_format_name(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return "PIPE_FORMAT_???";
    }
@@ -270,7 +269,6 @@ util_format_short_name(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return "???";
    }
@@ -298,7 +296,6 @@ util_format_is_compressed(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return FALSE;
    }
@@ -344,7 +341,6 @@ util_format_is_depth_or_stencil(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return FALSE;
    }
@@ -358,7 +354,6 @@ util_format_is_depth_and_stencil(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return FALSE;
    }
@@ -500,7 +495,6 @@ util_format_get_blocksizebits(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return 0;
    }
@@ -531,7 +525,6 @@ util_format_get_blockwidth(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return 1;
    }
@@ -544,7 +537,6 @@ util_format_get_blockheight(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   assert(desc);
    if (!desc) {
       return 1;
    }
@@ -599,12 +591,12 @@ util_format_get_component_bits(enum pipe_format format,
    const struct util_format_description *desc = util_format_description(format);
    enum util_format_colorspace desc_colorspace;
 
-   assert(format);
    if (!format) {
       return 0;
    }
 
-   assert(component < 4);
+   if (component >= 4)
+      return 0;
 
    /* Treat RGB and SRGB as equivalent. */
    if (colorspace == UTIL_FORMAT_COLORSPACE_SRGB) {
@@ -755,7 +747,6 @@ util_format_stencil_only(enum pipe_format format)
       return format;
 
    default:
-      assert(0);
       return PIPE_FORMAT_NONE;
    }
 }
