@@ -79,7 +79,7 @@ vkr_dispatch_vkAllocateCommandBuffers(struct vn_dispatch_context *dispatch,
    struct object_array arr;
 
    if (!pool) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -98,7 +98,7 @@ vkr_dispatch_vkFreeCommandBuffers(struct vn_dispatch_context *dispatch,
 
    /* args->pCommandBuffers is marked noautovalidity="true" */
    if (args->commandBufferCount && !args->pCommandBuffers) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -858,6 +858,24 @@ vkr_dispatch_vkCmdPushDescriptorSetKHR(UNUSED struct vn_dispatch_context *dispat
                 args->set, args->descriptorWriteCount, args->pDescriptorWrites);
 }
 
+static void
+vkr_dispatch_vkCmdSetColorWriteEnableEXT(
+   UNUSED struct vn_dispatch_context *dispatch,
+   struct vn_command_vkCmdSetColorWriteEnableEXT *args)
+{
+   VKR_CMD_CALL(CmdSetColorWriteEnableEXT, args, args->attachmentCount,
+                args->pColorWriteEnables);
+}
+
+static void
+vkr_dispatch_vkCmdSetVertexInputEXT(UNUSED struct vn_dispatch_context *dispatch,
+                                    struct vn_command_vkCmdSetVertexInputEXT *args)
+{
+   VKR_CMD_CALL(CmdSetVertexInputEXT, args, args->vertexBindingDescriptionCount,
+                args->pVertexBindingDescriptions, args->vertexAttributeDescriptionCount,
+                args->pVertexAttributeDescriptions);
+}
+
 void
 vkr_context_init_command_pool_dispatch(struct vkr_context *ctx)
 {
@@ -1002,4 +1020,11 @@ vkr_context_init_command_buffer_dispatch(struct vkr_context *ctx)
    /* VK_KHR_push_descriptor */
    dispatch->dispatch_vkCmdPushDescriptorSetKHR = vkr_dispatch_vkCmdPushDescriptorSetKHR;
    dispatch->dispatch_vkCmdPushDescriptorSetWithTemplateKHR = NULL;
+
+   /* VK_EXT_color_write_enable */
+   dispatch->dispatch_vkCmdSetColorWriteEnableEXT =
+      vkr_dispatch_vkCmdSetColorWriteEnableEXT;
+
+   /* VK_EXT_vertex_input_dynamic_state */
+   dispatch->dispatch_vkCmdSetVertexInputEXT = vkr_dispatch_vkCmdSetVertexInputEXT;
 }
