@@ -7,7 +7,6 @@
 
 #include "venus-protocol/vn_protocol_renderer_dispatches.h"
 #include "venus-protocol/vn_protocol_renderer_transport.h"
-#include "vrend_iov.h"
 
 #include "vkr_context.h"
 #include "vkr_ring.h"
@@ -18,17 +17,17 @@ vkr_dispatch_vkSetReplyCommandStreamMESA(
    struct vn_command_vkSetReplyCommandStreamMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
-   struct vkr_resource_attachment *att;
-
-   att = vkr_context_get_resource(ctx, args->pStream->resourceId);
-   if (!att) {
+   struct vkr_resource *res = vkr_context_get_resource(ctx, args->pStream->resourceId);
+   if (!res) {
       vkr_log("failed to set reply stream: invalid res_id %u", args->pStream->resourceId);
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
-   vkr_cs_encoder_set_stream(&ctx->encoder, att, args->pStream->offset,
-                             args->pStream->size);
+   struct vkr_cs_encoder *enc = (struct vkr_cs_encoder *)dispatch->encoder;
+   mtx_lock(&enc->mutex);
+   vkr_cs_encoder_set_stream(enc, res, args->pStream->offset, args->pStream->size);
+   mtx_unlock(&enc->mutex);
 }
 
 static void
@@ -36,65 +35,8 @@ vkr_dispatch_vkSeekReplyCommandStreamMESA(
    struct vn_dispatch_context *dispatch,
    struct vn_command_vkSeekReplyCommandStreamMESA *args)
 {
-   struct vkr_context *ctx = dispatch->data;
-   vkr_cs_encoder_seek_stream(&ctx->encoder, args->position);
-}
-
-static void *
-copy_command_stream(struct vkr_context *ctx, const VkCommandStreamDescriptionMESA *stream)
-{
-   struct vkr_resource_attachment *att;
-
-   att = vkr_context_get_resource(ctx, stream->resourceId);
-   if (!att) {
-      vkr_log("failed to copy command stream: invalid res_id %u", stream->resourceId);
-      return NULL;
-   }
-
-   /* seek to offset */
-   size_t iov_offset = stream->offset;
-   const struct iovec *iov = NULL;
-   for (int i = 0; i < att->iov_count; i++) {
-      if (iov_offset < att->iov[i].iov_len) {
-         iov = &att->iov[i];
-         break;
-      }
-      iov_offset -= att->iov[i].iov_len;
-   }
-   if (!iov) {
-      vkr_log("failed to copy command stream: invalid offset %zu", stream->offset);
-      return NULL;
-   }
-
-   /* XXX until the decoder supports scatter-gather and is robust enough,
-    * always make a copy in case the caller modifies the commands while we
-    * parse
-    */
-   uint8_t *data = malloc(stream->size);
-   if (!data) {
-      vkr_log("failed to copy command stream: malloc(%zu) failed", stream->size);
-      return NULL;
-   }
-
-   uint32_t copied = 0;
-   while (true) {
-      const size_t s = MIN2(stream->size - copied, iov->iov_len - iov_offset);
-      memcpy(data + copied, (const uint8_t *)iov->iov_base + iov_offset, s);
-
-      copied += s;
-      if (copied == stream->size) {
-         break;
-      } else if (iov == &att->iov[att->iov_count - 1]) {
-         vkr_log("failed to copy command stream: invalid size %zu", stream->size);
-         free(data);
-         return NULL;
-      }
-
-      iov++;
-      iov_offset = 0;
-   }
-
-   return data;
+   struct vkr_cs_encoder *enc = (struct vkr_cs_encoder *)dispatch->encoder;
+   vkr_cs_encoder_seek_stream(enc, args->position);
 }
 
 static void
@@ -103,17 +45,19 @@ vkr_dispatch_vkExecuteCommandStreamsMESA(
    struct vn_command_vkExecuteCommandStreamsMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
+   struct vkr_cs_decoder *dec = (struct vkr_cs_decoder *)dispatch->decoder;
+   struct vkr_cs_encoder *enc = (struct vkr_cs_encoder *)dispatch->encoder;
 
    if (unlikely(!args->streamCount)) {
       vkr_log("failed to execute command streams: no stream specified");
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
    /* note that nested vkExecuteCommandStreamsMESA is not allowed */
-   if (unlikely(!vkr_cs_decoder_push_state(&ctx->decoder))) {
+   if (unlikely(!vkr_cs_decoder_push_state(dec))) {
       vkr_log("failed to execute command streams: nested execution");
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -121,71 +65,76 @@ vkr_dispatch_vkExecuteCommandStreamsMESA(
       const VkCommandStreamDescriptionMESA *stream = &args->pStreams[i];
 
       if (args->pReplyPositions)
-         vkr_cs_encoder_seek_stream(&ctx->encoder, args->pReplyPositions[i]);
+         vkr_cs_encoder_seek_stream(enc, args->pReplyPositions[i]);
 
       if (!stream->size)
          continue;
 
-      void *data = copy_command_stream(ctx, stream);
-      if (!data) {
-         vkr_cs_decoder_set_fatal(&ctx->decoder);
+      struct vkr_resource *res = vkr_context_get_resource(ctx, stream->resourceId);
+      if (!res) {
+         vkr_log("failed to execute command streams: invalid stream %u res_id %u", i,
+                 stream->resourceId);
+         vkr_context_set_fatal(ctx);
          break;
       }
 
-      vkr_cs_decoder_set_stream(&ctx->decoder, data, stream->size);
-      while (vkr_cs_decoder_has_command(&ctx->decoder)) {
-         vn_dispatch_command(&ctx->dispatch);
-         if (vkr_cs_decoder_get_fatal(&ctx->decoder))
+      if (unlikely(stream->size > res->size ||
+                   stream->offset > res->size - stream->size)) {
+         vkr_log("failed to execute command streams: invalid stream %u res_id %u", i,
+                 stream->resourceId);
+         vkr_context_set_fatal(ctx);
+         break;
+      }
+
+      vkr_cs_decoder_set_stream(dec, res->u.data + stream->offset, stream->size);
+      while (vkr_cs_decoder_has_command(dec)) {
+         vn_dispatch_command(dispatch);
+         if (vkr_context_get_fatal(ctx))
             break;
       }
 
-      free(data);
-
-      if (vkr_cs_decoder_get_fatal(&ctx->decoder))
+      if (vkr_context_get_fatal(ctx))
          break;
    }
 
-   vkr_cs_decoder_pop_state(&ctx->decoder);
+   vkr_cs_decoder_pop_state(dec);
 }
 
 static struct vkr_ring *
 lookup_ring(struct vkr_context *ctx, uint64_t ring_id)
 {
    struct vkr_ring *ring;
+   mtx_lock(&ctx->ring_mutex);
    LIST_FOR_EACH_ENTRY (ring, &ctx->rings, head) {
-      if (ring->id == ring_id)
+      if (ring->id == ring_id) {
+         mtx_unlock(&ctx->ring_mutex);
          return ring;
+      }
    }
+   mtx_unlock(&ctx->ring_mutex);
    return NULL;
 }
 
 static bool
 vkr_ring_layout_init(struct vkr_ring_layout *layout,
-                     const struct vkr_resource_attachment *att,
+                     const struct vkr_resource *res,
                      const VkRingCreateInfoMESA *info)
 {
-   /* clang-format off */
    *layout = (struct vkr_ring_layout){
-      .attachment = att,
-      .head   = VKR_REGION_INIT(info->offset + info->headOffset, sizeof(uint32_t)),
-      .tail   = VKR_REGION_INIT(info->offset + info->tailOffset, sizeof(uint32_t)),
+      .resource = res,
+      .head = VKR_REGION_INIT(info->offset + info->headOffset, sizeof(uint32_t)),
+      .tail = VKR_REGION_INIT(info->offset + info->tailOffset, sizeof(uint32_t)),
       .status = VKR_REGION_INIT(info->offset + info->statusOffset, sizeof(uint32_t)),
       .buffer = VKR_REGION_INIT(info->offset + info->bufferOffset, info->bufferSize),
-      .extra  = VKR_REGION_INIT(info->offset + info->extraOffset, info->extraSize),
+      .extra = VKR_REGION_INIT(info->offset + info->extraOffset, info->extraSize),
    };
 
    const struct vkr_region res_region = VKR_REGION_INIT(info->offset, info->size);
    const struct vkr_region *regions[] = {
-      &layout->head,
-      &layout->tail,
-      &layout->status,
-      &layout->buffer,
-      &layout->extra,
+      &layout->head, &layout->tail, &layout->status, &layout->buffer, &layout->extra,
    };
-   /* clang-format on */
 
-   const struct vkr_region res_size =
-      VKR_REGION_INIT(0, vrend_get_iovec_size(att->iov, att->iov_count));
+   const struct vkr_region res_size = VKR_REGION_INIT(0, res->size);
    if (!vkr_region_is_valid(&res_region) || !vkr_region_is_within(&res_region, &res_size))
       return false;
 
@@ -233,36 +182,84 @@ vkr_ring_layout_init(struct vkr_ring_layout *layout,
    return true;
 }
 
+static inline bool
+is_dispatched_from_vkr_context(const struct vn_dispatch_context *dispatch)
+{
+   /* this is for -Wgnu-statement-expression-from-macro-expansion */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
+   return dispatch->data == container_of(dispatch, struct vkr_context, dispatch);
+#pragma GCC diagnostic pop
+}
+
 static void
 vkr_dispatch_vkCreateRingMESA(struct vn_dispatch_context *dispatch,
                               struct vn_command_vkCreateRingMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
    const VkRingCreateInfoMESA *info = args->pCreateInfo;
-   const struct vkr_resource_attachment *att;
-   struct vkr_ring *ring;
 
-   att = vkr_context_get_resource(ctx, info->resourceId);
-   if (!att) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   const struct vkr_resource *res = vkr_context_get_resource(ctx, info->resourceId);
+   if (!res) {
+      vkr_context_set_fatal(ctx);
       return;
    }
 
    struct vkr_ring_layout layout;
-   if (!vkr_ring_layout_init(&layout, att, info)) {
+   if (!vkr_ring_layout_init(&layout, res, info)) {
       vkr_log("vkCreateRingMESA supplied with invalid buffer layout parameters");
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
-   ring = vkr_ring_create(&layout, &ctx->base, info->idleTimeout);
+   struct vkr_ring *ring = vkr_ring_create(&layout, ctx, info->idleTimeout);
    if (!ring) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
    ring->id = args->ring;
+
+   mtx_lock(&ctx->ring_mutex);
    list_addtail(&ring->head, &ctx->rings);
+   mtx_unlock(&ctx->ring_mutex);
+
+   const VkRingMonitorInfoMESA *monitor_info =
+      vkr_find_struct(info->pNext, VK_STRUCTURE_TYPE_RING_MONITOR_INFO_MESA);
+   if (monitor_info) {
+      if (!monitor_info->maxReportingPeriodMicroseconds) {
+         vkr_log("invalid ring reporting period");
+         vkr_context_set_fatal(ctx);
+         return;
+      }
+
+      /* Start the ring monitoring thread or update the reporting rate of the running
+       * thread to the smallest maxReportingPeriodMicroseconds recieved so far, and wake
+       * it to begin reporting at the faster rate before the first driver check occurs.
+       */
+      if (!ctx->ring_monitor.started) {
+         if (!vkr_context_ring_monitor_init(
+                ctx, monitor_info->maxReportingPeriodMicroseconds)) {
+            vkr_context_set_fatal(ctx);
+            return;
+         }
+      } else if (monitor_info->maxReportingPeriodMicroseconds <
+                 ctx->ring_monitor.report_period_us) {
+         mtx_lock(&ctx->ring_monitor.mutex);
+         ctx->ring_monitor.report_period_us =
+            monitor_info->maxReportingPeriodMicroseconds;
+         cnd_signal(&ctx->ring_monitor.cond);
+         mtx_unlock(&ctx->ring_monitor.mutex);
+      }
+
+      ring->monitor = true;
+   }
 
    vkr_ring_start(ring);
 }
@@ -272,13 +269,22 @@ vkr_dispatch_vkDestroyRingMESA(struct vn_dispatch_context *dispatch,
                                struct vn_command_vkDestroyRingMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
-   struct vkr_ring *ring = lookup_ring(ctx, args->ring);
-   if (!ring || !vkr_ring_stop(ring)) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
+   struct vkr_ring *ring = lookup_ring(ctx, args->ring);
+   if (!ring || !vkr_ring_stop(ring)) {
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   mtx_lock(&ctx->ring_mutex);
    vkr_ring_destroy(ring);
+   mtx_unlock(&ctx->ring_mutex);
 }
 
 static void
@@ -286,9 +292,16 @@ vkr_dispatch_vkNotifyRingMESA(struct vn_dispatch_context *dispatch,
                               struct vn_command_vkNotifyRingMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
+
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
    struct vkr_ring *ring = lookup_ring(ctx, args->ring);
    if (!ring) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -300,37 +313,84 @@ vkr_dispatch_vkWriteRingExtraMESA(struct vn_dispatch_context *dispatch,
                                   struct vn_command_vkWriteRingExtraMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
+
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
    struct vkr_ring *ring = lookup_ring(ctx, args->ring);
    if (!ring) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
    if (!vkr_ring_write_extra(ring, args->offset, args->value))
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
 }
 
 static void
-vkr_dispatch_vkGetVenusExperimentalFeatureData100000MESA(
-   UNUSED struct vn_dispatch_context *dispatch,
-   struct vn_command_vkGetVenusExperimentalFeatureData100000MESA *args)
+vkr_dispatch_vkSubmitVirtqueueSeqnoMESA(struct vn_dispatch_context *dispatch,
+                                        struct vn_command_vkSubmitVirtqueueSeqnoMESA *args)
 {
-   const VkVenusExperimentalFeatures100000MESA features = {
-      .memoryResourceAllocationSize = VK_TRUE,
-      .globalFencing = VK_FALSE,
-      .largeRing = VK_TRUE,
-      .syncFdFencing = VK_TRUE,
-   };
+   struct vkr_context *ctx = dispatch->data;
 
-   vn_replace_vkGetVenusExperimentalFeatureData100000MESA_args_handle(args);
-
-   if (!args->pData) {
-      *args->pDataSize = sizeof(features);
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
-   *args->pDataSize = MIN2(*args->pDataSize, sizeof(features));
-   memcpy(args->pData, &features, *args->pDataSize);
+   struct vkr_ring *ring = lookup_ring(ctx, args->ring);
+   if (!ring) {
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   vkr_ring_submit_virtqueue_seqno(ring, args->seqno);
+}
+
+static void
+vkr_dispatch_vkWaitVirtqueueSeqnoMESA(struct vn_dispatch_context *dispatch,
+                                      struct vn_command_vkWaitVirtqueueSeqnoMESA *args)
+{
+   struct vkr_context *ctx = dispatch->data;
+   if (unlikely(is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on ring dispatch", __func__);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   /* this is for -Wgnu-statement-expression-from-macro-expansion */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
+   struct vkr_ring *ring = container_of(dispatch, struct vkr_ring, dispatch);
+#pragma GCC diagnostic pop
+   if (!vkr_ring_wait_virtqueue_seqno(ring, args->seqno))
+      vkr_context_set_fatal(ctx);
+}
+
+static void
+vkr_dispatch_vkWaitRingSeqnoMESA(struct vn_dispatch_context *dispatch,
+                                 struct vn_command_vkWaitRingSeqnoMESA *args)
+{
+   struct vkr_context *ctx = dispatch->data;
+
+   if (unlikely(!is_dispatched_from_vkr_context(dispatch))) {
+      vkr_log("%s must be called on context dispatch", __func__);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   struct vkr_ring *ring = lookup_ring(ctx, args->ring);
+   if (!ring) {
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   if (!vkr_context_wait_ring_seqno(ctx, ring, args->seqno))
+      vkr_context_set_fatal(ctx);
 }
 
 void
@@ -348,7 +408,8 @@ vkr_context_init_transport_dispatch(struct vkr_context *ctx)
    dispatch->dispatch_vkDestroyRingMESA = vkr_dispatch_vkDestroyRingMESA;
    dispatch->dispatch_vkNotifyRingMESA = vkr_dispatch_vkNotifyRingMESA;
    dispatch->dispatch_vkWriteRingExtraMESA = vkr_dispatch_vkWriteRingExtraMESA;
-
-   dispatch->dispatch_vkGetVenusExperimentalFeatureData100000MESA =
-      vkr_dispatch_vkGetVenusExperimentalFeatureData100000MESA;
+   dispatch->dispatch_vkSubmitVirtqueueSeqnoMESA =
+      vkr_dispatch_vkSubmitVirtqueueSeqnoMESA;
+   dispatch->dispatch_vkWaitVirtqueueSeqnoMESA = vkr_dispatch_vkWaitVirtqueueSeqnoMESA;
+   dispatch->dispatch_vkWaitRingSeqnoMESA = vkr_dispatch_vkWaitRingSeqnoMESA;
 }
