@@ -26,6 +26,7 @@
 
 #include <errno.h>
 
+#include "util/libsync.h"
 #include "util/os_misc.h"
 #include "util/u_hash_table.h"
 #include "util/u_pointer.h"
@@ -37,6 +38,10 @@ static void
 virgl_context_destroy_func(void *val)
 {
    struct virgl_context *ctx = val;
+
+   if (ctx->in_fence_fd >= 0)
+      close(ctx->in_fence_fd);
+
    ctx->destroy(ctx);
 }
 
@@ -99,3 +104,16 @@ virgl_context_foreach(const struct virgl_context_foreach_args *args)
                            virgl_context_foreach_func,
                            (void *)args);
 }
+
+int virgl_context_take_in_fence_fd(struct virgl_context *ctx)
+{
+   TRACE_FUNC();
+
+   if (ctx->in_fence_fd < 0)
+      return -1;
+
+   int ret = ctx->in_fence_fd;
+   ctx->in_fence_fd = -1;
+
+   return ret;
+}
