@@ -408,6 +408,7 @@ enum virgl_formats {
    VIRGL_FORMAT_R4G4B4A4_UNORM          = 319,
    VIRGL_FORMAT_R5G5B5A1_UNORM          = 320,
    VIRGL_FORMAT_R5G6B5_UNORM            = 321,
+   VIRGL_FORMAT_Y8_400_UNORM            = 322,
 
    VIRGL_FORMAT_MAX /* = PIPE_FORMAT_COUNT */,
 
@@ -467,6 +468,11 @@ enum virgl_formats {
 #define VIRGL_CAP_V2_SCANOUT_USES_GBM     (1 << 8)
 #define VIRGL_CAP_V2_SSO                  (1 << 9)
 #define VIRGL_CAP_V2_TEXTURE_SHADOW_LOD   (1 << 10)
+#define VIRGL_CAP_V2_VS_VERTEX_LAYER      (1 << 11)
+#define VIRGL_CAP_V2_VS_VIEWPORT_INDEX    (1 << 12)
+#define VIRGL_CAP_V2_PIPELINE_STATISTICS_QUERY (1 << 13)
+#define VIRGL_CAP_V2_DRAW_PARAMETERS      (1 << 14)
+#define VIRGL_CAP_V2_GROUP_VOTE           (1 << 15)
 
 /* virgl bind flags - these are compatible with mesa 10.5 gallium.
  * but are fixed, no other should be passed to virgl either.
@@ -648,6 +654,8 @@ struct virgl_caps_v2 {
         uint32_t num_video_caps;
         struct virgl_video_caps video_caps[32];
         uint32_t max_uniform_block_size;
+        uint32_t max_tcs_outputs;
+        uint32_t max_tes_outputs;
 };
 
 union virgl_caps {
@@ -681,6 +689,20 @@ enum virgl_ctx_errors {
         VIRGL_ERROR_CTX_ILLEGAL_PROGRAM_PIPELINE,
 };
 
+enum virgl_statistics_query_index {
+   VIRGL_STAT_QUERY_IA_VERTICES = 0,
+   VIRGL_STAT_QUERY_IA_PRIMITIVES = 1,
+   VIRGL_STAT_QUERY_VS_INVOCATIONS = 2,
+   VIRGL_STAT_QUERY_GS_INVOCATIONS = 3,
+   VIRGL_STAT_QUERY_GS_PRIMITIVES = 4,
+   VIRGL_STAT_QUERY_C_INVOCATIONS = 5,
+   VIRGL_STAT_QUERY_C_PRIMITIVES = 6,
+   VIRGL_STAT_QUERY_PS_INVOCATIONS = 7,
+   VIRGL_STAT_QUERY_HS_INVOCATIONS = 8,
+   VIRGL_STAT_QUERY_DS_INVOCATIONS = 9,
+   VIRGL_STAT_QUERY_CS_INVOCATIONS = 10,
+};
+
 /**
  * Flags for the driver about resource behaviour:
  */
