@@ -36,6 +36,7 @@
 
 #include "util/os_misc.h"
 #include "util/u_pointer.h"
+#include "util/u_string.h"
 
 #include <assert.h>
 #include <stdarg.h>
@@ -49,6 +50,13 @@
 #include <vperfetto-min.h>
 #endif
 
+#if ENABLE_TRACING == TRACE_WITH_SYSPROF
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
+#include <sysprof-capture.h>
+#pragma GCC diagnostic pop
+#endif
+
 #if ENABLE_TRACING == TRACE_WITH_STDERR
 #include <stdio.h>
 #endif
@@ -112,8 +120,28 @@ void flush_eventfd(int fd)
     } while ((len == -1 && errno == EINTR) || len == sizeof(value));
 }
 
+const struct log_levels_lut {
+   char *name;
+   enum virgl_log_level_flags log_level;
+} log_levels_table[] = {
+   {"debug", VIRGL_LOG_LEVEL_DEBUG},
+   {"info", VIRGL_LOG_LEVEL_INFO},
+   {"warning", VIRGL_LOG_LEVEL_WARNING},
+   {"error", VIRGL_LOG_LEVEL_ERROR},
+   { NULL, 0 },
+};
+
+#ifndef NDEBUG
+static enum virgl_log_level_flags virgl_log_level = VIRGL_LOG_LEVEL_WARNING;
+#else
+static enum virgl_log_level_flags virgl_log_level = VIRGL_LOG_LEVEL_ERROR;
+#endif
+static bool virgl_log_level_initialized = false;
+
 static
-void virgl_default_logger(const char *fmt, va_list va)
+void virgl_default_logger(UNUSED enum virgl_log_level_flags log_level,
+                          const char *message,
+                          UNUSED void* user_data)
 {
    static FILE* fp = NULL;
    if (NULL == fp) {
@@ -141,35 +169,87 @@ void virgl_default_logger(const char *fmt, va_list va)
             fp = stderr;
       }
    }
-   vfprintf(fp, fmt, va);
+
+   if (!virgl_log_level_initialized) {
+      const char* log_level_env = getenv("VIRGL_LOG_LEVEL");
+      if (log_level_env != NULL && log_level_env[0] != '\0') {
+         int log_index = 0;
+         const struct log_levels_lut *lut = &log_levels_table[0];
+         while (lut->name) {
+            if (!strcmp(lut->name, log_level_env)) {
+               virgl_log_level = lut->log_level;
+               break;
+            }
+
+            lut = &log_levels_table[++log_index];
+         }
+
+         if (!lut->name)
+            fprintf(fp, "Unknown log level %s requested\n", log_level_env);
+      }
+
+      virgl_log_level_initialized = true;
+   }
+
+   if (log_level < virgl_log_level)
+      return;
+
+   fprintf(fp, "%s", message);
    fflush(fp);
 }
 
-static
-void virgl_null_logger(UNUSED const char *fmt, UNUSED va_list va)
+void virgl_override_log_level(enum virgl_log_level_flags log_level)
 {
+   virgl_log_level = log_level;
+   virgl_log_level_initialized = true;
 }
 
-static virgl_debug_callback_type virgl_logger = virgl_default_logger;
+static struct {
+   virgl_log_callback_type log_cb;
+   virgl_free_data_callback_type free_data_cb;
+   void *user_data;
+} virgl_log_data = { virgl_default_logger, NULL, NULL };
 
-virgl_debug_callback_type virgl_log_set_logger(virgl_debug_callback_type logger)
+void virgl_log_set_handler(virgl_log_callback_type log_cb,
+                           void *user_data,
+                           virgl_free_data_callback_type free_data_cb)
 {
-   virgl_debug_callback_type old = virgl_logger;
+   if (virgl_log_data.free_data_cb)
+      virgl_log_data.free_data_cb(virgl_log_data.user_data);
 
-   /* virgl_null_logger is internal */
-   if (old == virgl_null_logger)
-      old = NULL;
-   if (!logger)
-      logger = virgl_null_logger;
-
-   virgl_logger = logger;
-   return old;
+   virgl_log_data.log_cb = log_cb;
+   virgl_log_data.free_data_cb = free_data_cb;
+   virgl_log_data.user_data = user_data;
 }
 
-void virgl_logv(const char *fmt, va_list va)
+void virgl_logv(enum virgl_log_level_flags log_level, const char *fmt, va_list va)
 {
-   assert(virgl_logger);
-   virgl_logger(fmt, va);
+   char *str = NULL;
+
+   if (!virgl_log_data.log_cb)
+      return;
+
+   if (vasprintf(&str, fmt, va) < 0)
+      return;
+
+   virgl_log_data.log_cb(log_level, str, virgl_log_data.user_data);
+   free (str);
+}
+
+void virgl_prefixed_logv(const char *domain,
+                         enum virgl_log_level_flags log_level,
+                         const char *fmt,
+                         va_list va)
+{
+   char *prefixed_fmt = NULL;
+
+   assert(strchr(domain,'%') == NULL);
+
+   if (asprintf(&prefixed_fmt, "%s: %s", domain, fmt) < 0)
+      return;
+
+   virgl_logv(log_level, prefixed_fmt, va);
+   free (prefixed_fmt);
 }
 
 #if ENABLE_TRACING == TRACE_WITH_PERCETTO
@@ -183,7 +263,7 @@ void trace_init(void)
 
 #if ENABLE_TRACING == TRACE_WITH_PERFETTO
 static void on_tracing_state_change(bool enabled) {
-    virgl_log("%s: tracing state change: %d\n", __func__, enabled);
+    virgl_debug("%s: tracing state change: %d\n", __func__, enabled);
 }
 
 void trace_init(void)
@@ -198,26 +278,58 @@ void trace_init(void)
    vperfetto_min_startTracing(&config);
 }
 
-const char *trace_begin(const char *scope)
+void *trace_begin(const char *scope)
 {
    vperfetto_min_beginTrackEvent_VMM(scope);
-   return scope;
+   return NULL;
 }
 
-void trace_end(const char **dummy)
+void trace_end(void **dummy)
 {
    (void)dummy;
    vperfetto_min_endTrackEvent_VMM();
 }
 #endif
 
+#if ENABLE_TRACING == TRACE_WITH_SYSPROF
+struct virgl_sysprof_entry {
+   SysprofTimeStamp begin;
+   /* SysprofCaptureMark itself limits it to 40 characters */
+   char name[40];
+};
+
+void trace_init(void)
+{
+}
+
+void *trace_begin(const char *scope)
+{
+   struct virgl_sysprof_entry *trace = malloc(sizeof (struct virgl_sysprof_entry));
+   trace->begin = SYSPROF_CAPTURE_CURRENT_TIME;
+   snprintf(trace->name, sizeof(trace->name), "%s", scope);
+
+   return trace;
+}
+
+void trace_end(void **func_name)
+{
+   struct virgl_sysprof_entry *trace = (struct virgl_sysprof_entry *)*func_name;
+   sysprof_collector_mark(trace->begin,
+                          SYSPROF_CAPTURE_CURRENT_TIME - trace->begin,
+                          "virglrenderer",
+                          trace->name,
+                          NULL);
+   free(trace);
+}
+#endif
+
 #if ENABLE_TRACING == TRACE_WITH_STDERR
 static int nesting_depth = 0;
 void trace_init(void)
 {
 }
 
-const char *trace_begin(const char *scope)
+void *trace_begin(const char *scope)
 {
    for (int i = 0; i < nesting_depth; ++i)
       fprintf(stderr, "  ");
@@ -225,14 +337,14 @@ const char *trace_begin(const char *scope)
    fprintf(stderr, "ENTER:%s\n", scope);
    nesting_depth++;
 
-   return scope;
+   return (void *)scope;
 }
 
-void trace_end(const char **func_name)
+void trace_end(void **func_name)
 {
    --nesting_depth;
    for (int i = 0; i < nesting_depth; ++i)
       fprintf(stderr, "  ");
-   fprintf(stderr, "LEAVE %s\n", *func_name);
+   fprintf(stderr, "LEAVE %s\n", (const char *) *func_name);
 }
 #endif
