@@ -31,22 +31,30 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
+#ifndef _WIN32
 #include <sys/mman.h>
+#else
+#include "mman_win32.h"
+#endif
 
 #include "pipe/p_state.h"
 #include "util/u_format.h"
 #include "util/u_math.h"
 #include "vkr_allocator.h"
-#include "vkr_renderer.h"
 #include "drm_renderer.h"
 #include "vrend_renderer.h"
 #include "proxy/proxy_renderer.h"
 #include "vrend_winsys.h"
 
+#ifndef WIN32
+#include "util/libsync.h"
+#endif
+
 #include "virglrenderer.h"
 #include "virglrenderer_hw.h"
 
 #include "virgl_context.h"
+#include "virgl_fence.h"
 #include "virgl_resource.h"
 #include "virgl_util.h"
 
@@ -60,9 +68,10 @@ struct global_state {
    bool context_initialized;
    bool winsys_initialized;
    bool vrend_initialized;
-   bool vkr_initialized;
    bool proxy_initialized;
    bool external_winsys_initialized;
+   bool drm_initialized;
+   bool fence_initialized;
 };
 
 static struct global_state state;
@@ -173,11 +182,12 @@ void virgl_renderer_fill_caps(uint32_t set, uint32_t version,
          vrend_renderer_fill_caps(set, version, (union virgl_caps *)caps);
       break;
    case VIRGL_RENDERER_CAPSET_VENUS:
-      if (state.vkr_initialized)
-         vkr_get_capset(caps);
+      if (state.proxy_initialized)
+         proxy_get_capset(set, caps);
       break;
    case VIRGL_RENDERER_CAPSET_DRM:
-      drm_renderer_capset(caps);
+      if (state.drm_initialized)
+         drm_renderer_capset(caps);
       break;
    default:
       break;
@@ -227,14 +237,13 @@ int virgl_renderer_context_create_with_flags(uint32_t ctx_id,
       ctx = vrend_renderer_context_create(ctx_id, nlen, name);
       break;
    case VIRGL_RENDERER_CAPSET_VENUS:
-      if (state.proxy_initialized)
-         ctx = proxy_context_create(ctx_id, ctx_flags, nlen, name);
-      else if (state.vkr_initialized)
-         ctx = vkr_context_create(nlen, name);
-      else
+      if (!state.proxy_initialized)
          return EINVAL;
+      ctx = proxy_context_create(ctx_id, ctx_flags, nlen, name);
       break;
    case VIRGL_RENDERER_CAPSET_DRM:
+      if (!state.drm_initialized)
+         return EINVAL;
       ctx = drm_renderer_create(nlen, name);
       break;
    default:
@@ -245,6 +254,7 @@ int virgl_renderer_context_create_with_flags(uint32_t ctx_id,
       return ENOMEM;
 
    ctx->ctx_id = ctx_id;
+   ctx->in_fence_fd = -1;
    ctx->capset_id = capset_id;
    ctx->fence_retire = per_context_fence_retire;
 
@@ -461,25 +471,78 @@ void virgl_renderer_ctx_detach_resource(int ctx_id, int res_handle)
    ctx->detach_resource(ctx, res);
 }
 
-int virgl_renderer_resource_get_info(int res_handle,
-                                     struct virgl_renderer_resource_info *info)
+static int virgl_renderer_resource_get_info_common(int res_handle,
+                                                   struct virgl_renderer_resource_info *info,
+                                                   UNUSED void **d3d_tex2d)
 {
+   int ret = 0;
+
    TRACE_FUNC();
    struct virgl_resource *res = virgl_resource_lookup(res_handle);
 
-   if (!res || !res->pipe_resource)
+   if (!res)
       return EINVAL;
    if (!info)
       return EINVAL;
 
+   info->handle = res_handle;
+   info->fd = res->fd;
+
+   if (!res->pipe_resource)
+      return 0;
+
    vrend_renderer_resource_get_info(res->pipe_resource,
                                     (struct vrend_renderer_resource_info *)info);
-   info->handle = res_handle;
+
+#ifdef WIN32
+   if (d3d_tex2d)
+      ret = vrend_renderer_resource_d3d11_texture2d(res->pipe_resource, d3d_tex2d);
+#endif
+
+   return ret;
+}
+
+int virgl_renderer_resource_get_info(int res_handle,
+                                     struct virgl_renderer_resource_info *info)
+{
+   TRACE_FUNC();
+   int ret;
+
+   if ((ret = virgl_renderer_resource_get_info_common(res_handle, info, NULL)) != 0)
+       return ret;
 
    if (state.winsys_initialized) {
-      return vrend_winsys_get_fourcc_for_texture(info->tex_id,
-                                                 info->virgl_format,
-                                                 &info->drm_fourcc);
+      return vrend_winsys_get_attrs_for_texture(info->tex_id,
+                                                info->virgl_format,
+                                                &info->drm_fourcc,
+                                                NULL,
+                                                NULL,
+                                                NULL);
+   }
+
+   return 0;
+}
+
+int virgl_renderer_resource_get_info_ext(int res_handle,
+                                         struct virgl_renderer_resource_info_ext *info_ext)
+{
+   TRACE_FUNC();
+   int ret;
+
+   if ((ret = virgl_renderer_resource_get_info_common(res_handle,
+                                                      &info_ext->base,
+                                                      &info_ext->d3d_tex2d)) != 0)
+      return ret;
+
+   info_ext->version = VIRGL_RENDERER_RESOURCE_INFO_EXT_VERSION;
+
+   if (state.winsys_initialized) {
+      return vrend_winsys_get_attrs_for_texture(info_ext->base.tex_id,
+                                                info_ext->base.virgl_format,
+                                                &info_ext->base.drm_fourcc,
+                                                &info_ext->has_dmabuf_export,
+                                                &info_ext->planes,
+                                                &info_ext->modifiers);
    }
 
    return 0;
@@ -498,7 +561,7 @@ void virgl_renderer_get_cap_set(uint32_t cap_set, uint32_t *max_ver,
       break;
    case VIRGL_RENDERER_CAPSET_VENUS:
       *max_ver = 0;
-      *max_size = vkr_get_capset(NULL);
+      *max_size = proxy_get_capset(cap_set, NULL);
       break;
    case VIRGL_RENDERER_CAPSET_DRM:
       *max_ver = 0;
@@ -539,8 +602,9 @@ static virgl_renderer_gl_context create_gl_context(int scanout_idx, struct virgl
    if (state.winsys_initialized)
       return vrend_winsys_create_context(param);
 
-   vparam.version = 1;
+   vparam.version = 2;
    vparam.shared = param->shared;
+   vparam.compat_ctx = param->compat_ctx;
    vparam.major_ver = param->major_ver;
    vparam.minor_ver = param->minor_ver;
    return state.cbs->create_gl_context(state.cookie, scanout_idx, &vparam);
@@ -558,10 +622,62 @@ static void destroy_gl_context(virgl_renderer_gl_context ctx)
 
 static int make_current(virgl_renderer_gl_context ctx)
 {
+   int ret;
+
    if (state.winsys_initialized)
       return vrend_winsys_make_context_current(ctx);
 
-   return state.cbs->make_current(state.cookie, 0, ctx);
+   ret = state.cbs->make_current(state.cookie, 0, ctx);
+   if (ret && state.cbs->version >= 4) {
+      virgl_error("%s: Error switching context: %d\n", __func__, ret);
+      assert(!ret && "Failed to switch GL context");
+      return -1;
+   }
+
+   return 0;
+}
+
+static virgl_renderer_gl_context create_gl_context_surfaceless(int scanout_idx, struct virgl_gl_ctx_param *param)
+{
+   struct virgl_renderer_gl_ctx_param vparam;
+
+   if (state.winsys_initialized || state.external_winsys_initialized)
+      return vrend_winsys_create_context(param);
+
+   vparam.version = 2;
+   vparam.shared = param->shared;
+   vparam.major_ver = param->major_ver;
+   vparam.minor_ver = param->minor_ver;
+   vparam.compat_ctx = param->compat_ctx;
+   return state.cbs->create_gl_context(state.cookie, scanout_idx, &vparam);
+}
+
+static void destroy_gl_context_surfaceless(virgl_renderer_gl_context ctx)
+{
+   if (state.winsys_initialized || state.external_winsys_initialized) {
+      vrend_winsys_destroy_context(ctx);
+      return;
+   }
+
+   state.cbs->destroy_gl_context(state.cookie, ctx);
+}
+
+static int make_current_surfaceless(virgl_renderer_gl_context ctx)
+{
+   int ret;
+
+   if (state.winsys_initialized || state.external_winsys_initialized)
+      return vrend_winsys_make_context_current(ctx);
+
+   ret = state.cbs->make_current(state.cookie, 0, ctx);
+   if (ret && state.cbs->version >= 4) {
+      virgl_error("%s: Error switching surfaceless context: %d\n",
+                  __func__, ret);
+      assert(!ret && "Failed to switch GL context");
+      return -1;
+   }
+
+   return 0;
 }
 
 static int get_drm_fd(void)
@@ -578,6 +694,9 @@ static const struct vrend_if_cbs vrend_cbs = {
    destroy_gl_context,
    make_current,
    get_drm_fd,
+   create_gl_context_surfaceless,
+   destroy_gl_context_surfaceless,
+   make_current_surfaceless,
 };
 
 static int
@@ -611,7 +730,8 @@ virgl_context_foreach_retire_fences(struct virgl_context *ctx,
 {
    /* vrend contexts are polled explicitly by the caller */
    if (ctx->capset_id != VIRGL_RENDERER_CAPSET_VIRGL &&
-       ctx->capset_id != VIRGL_RENDERER_CAPSET_VIRGL2)
+       ctx->capset_id != VIRGL_RENDERER_CAPSET_VIRGL2 &&
+       !(state.flags & VIRGL_RENDERER_ASYNC_FENCE_CB))
    {
       assert(ctx->retire_fences);
       ctx->retire_fences(ctx);
@@ -645,19 +765,20 @@ void virgl_renderer_cleanup(UNUSED void *cookie)
    if (state.proxy_initialized)
       proxy_renderer_fini();
 
-   if (state.vkr_initialized) {
-      vkr_renderer_fini();
-      /* vkr_allocator_init is called on-demand upon the first map */
-      vkr_allocator_fini();
-   }
-
    if (state.vrend_initialized)
       vrend_renderer_fini();
 
+   if (state.fence_initialized)
+      virgl_fence_table_cleanup();
+
    if (state.winsys_initialized || state.external_winsys_initialized)
       vrend_winsys_cleanup();
 
-   drm_renderer_fini();
+   if (state.drm_initialized)
+      drm_renderer_fini();
+
+   /* vkr_allocator_init is called on-demand upon the first map */
+   vkr_allocator_fini();
 
    memset(&state, 0, sizeof(state));
 }
@@ -768,6 +889,10 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
          renderer_flags |= VREND_USE_EXTERNAL_BLOB;
       if (flags & VIRGL_RENDERER_USE_VIDEO)
          renderer_flags |= VREND_USE_VIDEO;
+      if (flags & VIRGL_RENDERER_D3D11_SHARE_TEXTURE)
+         renderer_flags |= VREND_D3D11_SHARE_TEXTURE;
+      if (flags & VIRGL_RENDERER_COMPAT_PROFILE)
+         renderer_flags |= VREND_USE_COMPAT_CONTEXT;
 
       ret = vrend_renderer_init(&vrend_cbs, renderer_flags);
       if (ret)
@@ -775,21 +900,6 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
       state.vrend_initialized = true;
    }
 
-   if (!state.vkr_initialized && (flags & VIRGL_RENDERER_VENUS)) {
-      uint32_t vkr_flags = 0;
-      if (flags & VIRGL_RENDERER_THREAD_SYNC)
-         vkr_flags |= VKR_RENDERER_THREAD_SYNC;
-      if (flags & VIRGL_RENDERER_ASYNC_FENCE_CB)
-         vkr_flags |= VKR_RENDERER_ASYNC_FENCE_CB;
-      if (flags & VIRGL_RENDERER_RENDER_SERVER)
-         vkr_flags |= VKR_RENDERER_RENDER_SERVER;
-
-      ret = vkr_renderer_init(vkr_flags);
-      if (ret)
-         goto fail;
-      state.vkr_initialized = true;
-   }
-
    if (!state.proxy_initialized && (flags & VIRGL_RENDERER_RENDER_SERVER)) {
       ret = proxy_renderer_init(&proxy_cbs, flags | VIRGL_RENDERER_NO_VIRGL);
       if (ret)
@@ -804,7 +914,17 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
       if (cbs->version >= 2 && cbs->get_drm_fd)
          drm_fd = cbs->get_drm_fd(cookie);
 
-      drm_renderer_init(drm_fd);
+      ret = drm_renderer_init(drm_fd);
+      if (ret)
+         goto fail;
+      state.drm_initialized = true;
+   }
+
+   if (!state.fence_initialized) {
+      ret = virgl_fence_table_init();
+      if (ret)
+         goto fail;
+      state.fence_initialized = true;
    }
 
    return 0;
@@ -845,13 +965,11 @@ void virgl_renderer_reset(void)
    if (state.proxy_initialized)
       proxy_renderer_reset();
 
-   if (state.vkr_initialized)
-      vkr_renderer_reset();
-
    if (state.vrend_initialized)
       vrend_renderer_reset();
 
-   drm_renderer_reset();
+   if (state.drm_initialized)
+      drm_renderer_reset();
 }
 
 int virgl_renderer_get_poll_fd(void)
@@ -863,9 +981,50 @@ int virgl_renderer_get_poll_fd(void)
    return -1;
 }
 
+static
+void virgl_null_logger(UNUSED const char *fmt, UNUSED va_list va)
+{
+}
+
+/* Compatibility layer for the virgl_set_debug_callback function */
+static inline void virgl_legacy_logger_wrapper(virgl_debug_callback_type cb,
+                                               const char *fmt,
+                                               ...)
+{
+   va_list va;
+   va_start(va, fmt);
+   cb(fmt, va);
+   va_end(va);
+}
+
+/* The logger need to be wrapped into a structure to be given as void* */
+struct virgl_legacy_logger_holder {
+   virgl_debug_callback_type logger;
+};
+
+static void virgl_legacy_logger(UNUSED enum virgl_log_level_flags log_level,
+                                const char *message,
+                                void* user_data)
+{
+   struct virgl_legacy_logger_holder *log_cb = user_data;
+   virgl_legacy_logger_wrapper(log_cb->logger, "%s", message);
+}
+
+static struct virgl_legacy_logger_holder legacy_logger = { virgl_null_logger };
+
 virgl_debug_callback_type virgl_set_debug_callback(virgl_debug_callback_type cb)
 {
-   return virgl_log_set_logger(cb);
+   virgl_debug_callback_type previous_cb = legacy_logger.logger;
+   legacy_logger.logger = cb;
+   virgl_log_set_handler(virgl_legacy_logger, &legacy_logger, NULL);
+   return previous_cb;
+}
+
+void virgl_set_log_callback(virgl_log_callback_type cb,
+                            void* user_data,
+                            virgl_free_data_callback_type free_user_data_cb)
+{
+   virgl_log_set_handler(cb, user_data, free_user_data_cb);
 }
 
 static int virgl_renderer_export_query(void *execute_args, uint32_t execute_size)
@@ -1016,7 +1175,7 @@ int virgl_renderer_resource_create_blob(const struct virgl_renderer_resource_cre
                                           blob.u.fd,
                                           args->iovecs,
                                           args->num_iovs,
-                                          &blob.opaque_fd_metadata);
+                                          &blob.vulkan_info);
       if (!res) {
          close(blob.u.fd);
          return -ENOMEM;
@@ -1062,7 +1221,11 @@ int virgl_renderer_resource_map(uint32_t res_handle, void **out_map, uint64_t *o
       case VIRGL_RESOURCE_FD_OPAQUE:
          ret = vkr_allocator_resource_map(res, &map, &map_size);
          break;
-      default:
+      case VIRGL_RESOURCE_OPAQUE_HANDLE:
+      case VIRGL_RESOURCE_FD_INVALID:
+         /* Avoid a default case so that -Wswitch will tell us at compile time
+          * if a new virgl resource type is added without being handled here.
+          */
          break;
       }
    }
@@ -1079,7 +1242,7 @@ int virgl_renderer_resource_map(uint32_t res_handle, void **out_map, uint64_t *o
 int virgl_renderer_resource_unmap(uint32_t res_handle)
 {
    TRACE_FUNC();
-   int ret;
+   int ret = 0;
    struct virgl_resource *res = virgl_resource_lookup(res_handle);
    if (!res || !res->mapped)
       return -EINVAL;
@@ -1089,12 +1252,17 @@ int virgl_renderer_resource_unmap(uint32_t res_handle)
    } else {
       switch (res->fd_type) {
       case VIRGL_RESOURCE_FD_DMABUF:
+      case VIRGL_RESOURCE_FD_SHM:
          ret = munmap(res->mapped, res->map_size);
          break;
       case VIRGL_RESOURCE_FD_OPAQUE:
          ret = vkr_allocator_resource_unmap(res);
          break;
-      default:
+      case VIRGL_RESOURCE_OPAQUE_HANDLE:
+      case VIRGL_RESOURCE_FD_INVALID:
+         /* Avoid a default case so that -Wswitch will tell us at compile time
+          * if a new virgl resource type is added without being handled here.
+          */
          ret = -EINVAL;
          break;
       }
@@ -1126,7 +1294,7 @@ virgl_renderer_resource_export_blob(uint32_t res_id, uint32_t *fd_type, int *fd)
    TRACE_FUNC();
    struct virgl_resource *res = virgl_resource_lookup(res_id);
    if (!res)
-      return EINVAL;
+      return -EINVAL;
 
    switch (virgl_resource_export_fd(res, fd)) {
    case VIRGL_RESOURCE_FD_DMABUF:
@@ -1138,8 +1306,12 @@ virgl_renderer_resource_export_blob(uint32_t res_id, uint32_t *fd_type, int *fd)
    case VIRGL_RESOURCE_FD_SHM:
       *fd_type = VIRGL_RENDERER_BLOB_FD_TYPE_SHM;
       break;
-   default:
-      return EINVAL;
+   case VIRGL_RESOURCE_OPAQUE_HANDLE:
+   case VIRGL_RESOURCE_FD_INVALID:
+      /* Avoid a default case so that -Wswitch will tell us at compile time if a
+       * new virgl resource type is added without being handled here.
+       */
+      return -EINVAL;
    }
 
    return 0;
@@ -1203,8 +1375,93 @@ virgl_renderer_resource_import_blob(const struct virgl_renderer_resource_import_
 }
 
 int
-virgl_renderer_export_fence(uint32_t client_fence_id, int *fd)
+virgl_renderer_export_fence(uint64_t client_fence_id, int *fd)
 {
    TRACE_FUNC();
-   return vrend_renderer_export_ctx0_fence(client_fence_id, fd);
+
+   /* transfers FD ownership to caller */
+   *fd = virgl_fence_get_fd(client_fence_id);
+   if (*fd >= 0)
+      return 0;
+
+   return -EINVAL;
+}
+
+int virgl_renderer_export_signalled_fence(void)
+{
+   TRACE_FUNC();
+
+   /* transfers FD ownership to caller, returns -1 on failure */
+   return virgl_fence_get_last_signalled_fence_fd();
+}
+
+static int virgl_renderer_context_attach_in_fence(struct virgl_context *ctx,
+                                                  uint64_t fence_id)
+{
+   int ret = -EINVAL;
+
+   /*
+    * FD will be -1 in two cases:
+    *
+    *    1. Fence was signalled and retired.
+    *    2. Fence ID is invalid. Virglrenderer doesn't take responsibility
+    *       for handling invalid fences and assumes that all supplied fence
+    *       IDs are always valid. It's caller's responsibility to validate
+    *       fence IDs.
+    */
+   int fd = virgl_fence_get_fd(fence_id);
+   if (fd < 0)
+      return 0;
+
+#ifndef WIN32
+   ret = sync_accumulate("virglrenderer", &ctx->in_fence_fd, fd);
+#endif
+   close(fd);
+
+   if (ret)
+      virgl_error("%s: sync_accumulate failed for fence_id=%" PRIu64 " err=%d\n",
+                  __func__, fence_id, ret);
+
+   return ret;
+}
+
+static int virgl_renderer_context_attach_in_fences(struct virgl_context *ctx,
+                                                   uint64_t *fence_ids,
+                                                   uint32_t num_fences)
+{
+   TRACE_FUNC();
+
+   if (!ctx->supports_fence_sharing)
+      return -EINVAL;
+
+   for (uint32_t i = 0; i < num_fences; i++) {
+      int ret = virgl_renderer_context_attach_in_fence(ctx, fence_ids[i]);
+      if (ret)
+         return ret;
+   }
+
+   return 0;
+}
+
+int virgl_renderer_submit_cmd2(void *buffer,
+                               int ctx_id,
+                               int ndw,
+                               uint64_t *in_fence_ids,
+                               uint32_t num_in_fences)
+{
+   TRACE_FUNC();
+   struct virgl_context *ctx = virgl_context_lookup(ctx_id);
+   if (!ctx)
+      return EINVAL;
+
+   if (ndw < 0 || (unsigned)ndw > UINT32_MAX / sizeof(uint32_t))
+      return EINVAL;
+
+   if (num_in_fences) {
+      int err = virgl_renderer_context_attach_in_fences(ctx, in_fence_ids, num_in_fences);
+      if (err)
+         return err;
+   }
+
+   return ctx->submit_cmd(ctx, buffer, ndw * sizeof(uint32_t));
 }
