@@ -280,6 +280,7 @@ static int vrend_decode_set_constant_buffer(struct vrend_context *ctx, const uin
 {
    uint32_t shader;
    int nc = (length - 2);
+   const float *data = NULL;
 
    if (length < 2)
       return EINVAL;
@@ -290,7 +291,10 @@ static int vrend_decode_set_constant_buffer(struct vrend_context *ctx, const uin
    if (shader >= PIPE_SHADER_TYPES)
       return EINVAL;
 
-   vrend_set_constants(ctx, shader, nc, get_buf_ptr(buf, VIRGL_SET_CONSTANT_BUFFER_DATA_START));
+   if (length > 2)
+      data = get_buf_ptr(buf, VIRGL_SET_CONSTANT_BUFFER_DATA_START);
+
+   vrend_set_constants(ctx, shader, nc, data);
    return 0;
 }
 
@@ -501,45 +505,36 @@ static int vrend_decode_create_blend(struct vrend_context *ctx, const uint32_t *
 static int vrend_decode_create_dsa(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint16_t length)
 {
    int i;
-   struct pipe_depth_stencil_alpha_state *dsa_state;
+   struct pipe_depth_stencil_alpha_state dsa_state;
    uint32_t tmp;
 
    if (length != VIRGL_OBJ_DSA_SIZE)
       return EINVAL;
 
-   dsa_state = CALLOC_STRUCT(pipe_depth_stencil_alpha_state);
-   if (!dsa_state)
-      return ENOMEM;
-
+   memset(&dsa_state, 0, sizeof(dsa_state));
    tmp = get_buf_entry(buf, VIRGL_OBJ_DSA_S0);
-   dsa_state->depth.enabled = tmp & 0x1;
-   dsa_state->depth.writemask = (tmp >> 1) & 0x1;
-   dsa_state->depth.func = (tmp >> 2) & 0x7;
+   dsa_state.depth.enabled = tmp & 0x1;
+   dsa_state.depth.writemask = (tmp >> 1) & 0x1;
+   dsa_state.depth.func = (tmp >> 2) & 0x7;
 
-   dsa_state->alpha.enabled = (tmp >> 8) & 0x1;
-   dsa_state->alpha.func = (tmp >> 9) & 0x7;
+   dsa_state.alpha.enabled = (tmp >> 8) & 0x1;
+   dsa_state.alpha.func = (tmp >> 9) & 0x7;
 
    for (i = 0; i < 2; i++) {
       tmp = get_buf_entry(buf, VIRGL_OBJ_DSA_S1 + i);
-      dsa_state->stencil[i].enabled = tmp & 0x1;
-      dsa_state->stencil[i].func = (tmp >> 1) & 0x7;
-      dsa_state->stencil[i].fail_op = (tmp >> 4) & 0x7;
-      dsa_state->stencil[i].zpass_op = (tmp >> 7) & 0x7;
-      dsa_state->stencil[i].zfail_op = (tmp >> 10) & 0x7;
-      dsa_state->stencil[i].valuemask = (tmp >> 13) & 0xff;
-      dsa_state->stencil[i].writemask = (tmp >> 21) & 0xff;
+      dsa_state.stencil[i].enabled = tmp & 0x1;
+      dsa_state.stencil[i].func = (tmp >> 1) & 0x7;
+      dsa_state.stencil[i].fail_op = (tmp >> 4) & 0x7;
+      dsa_state.stencil[i].zpass_op = (tmp >> 7) & 0x7;
+      dsa_state.stencil[i].zfail_op = (tmp >> 10) & 0x7;
+      dsa_state.stencil[i].valuemask = (tmp >> 13) & 0xff;
+      dsa_state.stencil[i].writemask = (tmp >> 21) & 0xff;
    }
 
    tmp = get_buf_entry(buf, VIRGL_OBJ_DSA_ALPHA_REF);
-   dsa_state->alpha.ref_value = uif(tmp);
+   dsa_state.alpha.ref_value = uif(tmp);
 
-   tmp = vrend_renderer_object_insert(ctx, dsa_state, handle,
-                                      VIRGL_OBJECT_DSA);
-   if (tmp == 0) {
-      FREE(dsa_state);
-      return ENOMEM;
-   }
-   return 0;
+   return vrend_create_dsa(ctx, handle, &dsa_state);
 }
 
 static int vrend_decode_create_rasterizer(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint16_t length)
@@ -610,15 +605,25 @@ static int vrend_decode_create_rasterizer(struct vrend_context *ctx, const uint3
 
 static int vrend_decode_create_surface_common(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint32_t sample_count)
 {
-   uint32_t res_handle, format, val0, val1;
+   uint32_t res_handle = get_buf_entry(buf, VIRGL_OBJ_SURFACE_RES_HANDLE);
+
+   struct vrend_resource *res = vrend_renderer_ctx_res_lookup(ctx, res_handle);
+   if (!res) {
+      vrend_report_context_error(ctx, VIRGL_ERROR_CTX_ILLEGAL_RESOURCE, res_handle);
+      return EINVAL;
+   }
+
+   enum virgl_formats format = get_buf_entry(buf, VIRGL_OBJ_SURFACE_FORMAT);
+
+   if (format >= PIPE_FORMAT_COUNT) {
+      return EINVAL;
+   }
 
-   res_handle = get_buf_entry(buf, VIRGL_OBJ_SURFACE_RES_HANDLE);
-   format = get_buf_entry(buf, VIRGL_OBJ_SURFACE_FORMAT);
    /* decide later if these are texture or buffer */
-   val0 = get_buf_entry(buf, VIRGL_OBJ_SURFACE_BUFFER_FIRST_ELEMENT);
-   val1 = get_buf_entry(buf, VIRGL_OBJ_SURFACE_BUFFER_LAST_ELEMENT);
+   uint32_t val0 = get_buf_entry(buf, VIRGL_OBJ_SURFACE_BUFFER_FIRST_ELEMENT);
+   uint32_t val1 = get_buf_entry(buf, VIRGL_OBJ_SURFACE_BUFFER_LAST_ELEMENT);
 
-   return vrend_create_surface(ctx, handle, res_handle, format, val0, val1, sample_count);
+   return vrend_create_surface(ctx, handle, res, format, val0, val1, sample_count);
 }
 
 static int vrend_decode_create_surface(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint16_t length)
@@ -640,17 +645,35 @@ static int vrend_decode_create_msaa_surface(struct vrend_context *ctx, const uin
 
 static int vrend_decode_create_sampler_view(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint16_t length)
 {
-   uint32_t res_handle, format, val0, val1, swizzle_packed;
-
    if (length != VIRGL_OBJ_SAMPLER_VIEW_SIZE)
       return EINVAL;
 
-   res_handle = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_RES_HANDLE);
-   format = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_FORMAT);
-   val0 = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_BUFFER_FIRST_ELEMENT);
-   val1 = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_BUFFER_LAST_ELEMENT);
-   swizzle_packed = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_SWIZZLE);
-   return vrend_create_sampler_view(ctx, handle, res_handle, format, val0, val1,swizzle_packed);
+   uint32_t res_handle = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_RES_HANDLE);
+   uint32_t format_data = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_FORMAT);
+
+   enum virgl_formats format = format_data & 0xffffff;
+   if (!format || format >= VIRGL_FORMAT_MAX) {
+      vrend_report_context_error(ctx, VIRGL_ERROR_CTX_ILLEGAL_FORMAT, format_data);
+      return EINVAL;
+   }
+
+   enum pipe_texture_target pipe_target = (format_data >> 24) & 0xff;
+   if (pipe_target >= PIPE_MAX_TEXTURE_TYPES) {
+      vrend_report_context_error(ctx, VIRGL_ERROR_CTX_ILLEGAL_SAMPLER_VIEW_TARGET,
+                                 format_data);
+      return EINVAL;
+   }
+
+   struct vrend_resource *res = vrend_renderer_ctx_res_lookup(ctx, res_handle);
+   if (!res) {
+      vrend_report_context_error(ctx, VIRGL_ERROR_CTX_ILLEGAL_RESOURCE, res_handle);
+      return EINVAL;
+   }
+
+   uint32_t val0 = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_BUFFER_FIRST_ELEMENT);
+   uint32_t val1 = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_BUFFER_LAST_ELEMENT);
+   uint32_t swizzle_packed = get_buf_entry(buf, VIRGL_OBJ_SAMPLER_VIEW_SWIZZLE);
+   return vrend_create_sampler_view(ctx, handle, res, format, pipe_target, val0, val1,swizzle_packed);
 }
 
 static int vrend_decode_create_sampler_state(struct vrend_context *ctx, const uint32_t *buf, uint32_t handle, uint16_t length)
@@ -852,7 +875,7 @@ static int vrend_decode_destroy_object(struct vrend_context *ctx, const uint32_t
 
    VREND_DEBUG_EXT(dbg_object, ctx,
                uint32_t obj = (get_buf_entry(buf, 0) >> 8) & 0xFF;
-               vrend_printf("  DESTROY %-17s handle:0x%x\n",
+               virgl_debug("  DESTROY %-17s handle:0x%x\n",
                        vrend_get_object_type_name(obj), handle));
 
    vrend_renderer_object_destroy(ctx, handle);
@@ -890,31 +913,33 @@ static int vrend_decode_set_blend_color(struct vrend_context *ctx, const uint32_
 
 static int vrend_decode_set_scissor_state(struct vrend_context *ctx, const uint32_t *buf, uint32_t length)
 {
-   struct pipe_scissor_state ss[PIPE_MAX_VIEWPORTS];
-   uint32_t temp;
-   int32_t num_scissor;
-   uint32_t start_slot;
-   int s;
    if (length < 1)
       return EINVAL;
 
    if ((length - 1) % 2)
       return EINVAL;
 
-   num_scissor = (length - 1) / 2;
+   const int32_t num_scissor = (length - 1) / 2;
    if (num_scissor > PIPE_MAX_VIEWPORTS)
       return EINVAL;
 
-   start_slot = get_buf_entry(buf, VIRGL_SET_SCISSOR_START_SLOT);
+   const uint32_t start_slot = get_buf_entry(buf, VIRGL_SET_SCISSOR_START_SLOT);
 
-   for (s = 0; s < num_scissor; s++) {
-      temp = get_buf_entry(buf, VIRGL_SET_SCISSOR_MINX_MINY(s));
-      ss[s].minx = temp & 0xffff;
-      ss[s].miny = (temp >> 16) & 0xffff;
+   if (start_slot >= PIPE_MAX_VIEWPORTS ||
+       start_slot + num_scissor > PIPE_MAX_VIEWPORTS) {
+      vrend_report_buffer_error(ctx, 0);
+      return EINVAL;
+   }
 
-      temp = get_buf_entry(buf, VIRGL_SET_SCISSOR_MAXX_MAXY(s));
-      ss[s].maxx = temp & 0xffff;
-      ss[s].maxy = (temp >> 16) & 0xffff;
+   struct pipe_scissor_state ss[PIPE_MAX_VIEWPORTS];
+   for (int s = 0; s < num_scissor; s++) {
+      const uint32_t minxy = get_buf_entry(buf, VIRGL_SET_SCISSOR_MINX_MINY(s));
+      ss[s].minx = minxy & 0xffff;
+      ss[s].miny = (minxy >> 16) & 0xffff;
+
+      const uint32_t maxxy = get_buf_entry(buf, VIRGL_SET_SCISSOR_MAXX_MAXY(s));
+      ss[s].maxx = maxxy & 0xffff;
+      ss[s].maxy = (maxxy >> 16) & 0xffff;
    }
 
    vrend_set_scissor_state(ctx, start_slot, num_scissor, ss);
@@ -1050,6 +1075,7 @@ static int vrend_decode_blit(struct vrend_context *ctx, const uint32_t *buf, uin
 
 static int vrend_decode_bind_sampler_states(struct vrend_context *ctx, const uint32_t *buf, uint32_t length)
 {
+   const uint32_t *handles = NULL;
    if (length < 2)
       return EINVAL;
 
@@ -1060,8 +1086,10 @@ static int vrend_decode_bind_sampler_states(struct vrend_context *ctx, const uin
    if (shader_type >= PIPE_SHADER_TYPES)
       return EINVAL;
 
-   vrend_bind_sampler_states(ctx, shader_type, start_slot, num_states,
-                             get_buf_ptr(buf, VIRGL_BIND_SAMPLER_STATES_S0_HANDLE));
+   if (num_states > 0)
+      handles = get_buf_ptr(buf, VIRGL_BIND_SAMPLER_STATES_S0_HANDLE);
+
+   vrend_bind_sampler_states(ctx, shader_type, start_slot, num_states, handles);
    return 0;
 }
 
@@ -1190,7 +1218,7 @@ static int vrend_decode_bind_shader(struct vrend_context *ctx, const uint32_t *b
 }
 
 static int vrend_decode_set_tess_state(struct vrend_context *ctx,
-				       const uint32_t *buf, uint32_t length)
+                                       const uint32_t *buf, uint32_t length)
 {
    float tess_factors[6];
    int i;
@@ -1359,7 +1387,7 @@ static int vrend_decode_texture_barrier(struct vrend_context *ctx, const uint32_
 static int vrend_decode_set_debug_mask(struct vrend_context *ctx, const uint32_t *buf, uint32_t length)
 {
    char *flagstring;
-   int slen = sizeof(uint32_t) * length;
+   size_t slen = sizeof(uint32_t) * (length - VIRGL_SET_DEBUG_FLAGSTRING_OFFSET);
 
    if (length < VIRGL_SET_DEBUG_FLAGS_MIN_SIZE)
       return EINVAL;
@@ -1614,7 +1642,7 @@ static int vrend_decode_get_memory_info(struct vrend_context *ctx, const uint32_
 
 static int vrend_decode_send_string_marker(struct vrend_context *ctx, const uint32_t *buf, uint32_t length)
 {
-   uint32_t buf_len = sizeof(uint32_t) * (length - 1);
+   size_t buf_len = sizeof(uint32_t) * (length - 1);
 
    if (length < VIRGL_SEND_STRING_MARKER_MIN_SIZE) {
       fprintf(stderr, "minimal command length not okay\n");
@@ -1623,7 +1651,7 @@ static int vrend_decode_send_string_marker(struct vrend_context *ctx, const uint
 
    uint32_t str_len = get_buf_entry(buf, VIRGL_SEND_STRING_MARKER_STRING_SIZE);
    if (str_len > buf_len) {
-       fprintf(stderr, "String len %u > buf_len %u\n", str_len, buf_len);
+       fprintf(stderr, "String len %u > buf_len %zu\n", str_len, buf_len);
        return EINVAL;
    }
 
@@ -1944,7 +1972,7 @@ static int vrend_decode_ctx_submit_cmd(struct virgl_context *ctx,
       if (!vrend_check_no_error(gdctx->grctx) && !ret)
          ret = EINVAL;
       if (ret) {
-         vrend_printf("context %d failed to dispatch %s: %d\n",
+         virgl_error("context %d failed to dispatch %s: %d\n",
                gdctx->base.ctx_id, vrend_get_comand_name(cmd), ret);
          if (ret == EINVAL)
             vrend_report_buffer_error(gdctx->grctx, *buf);
