@@ -66,8 +66,6 @@
  */
 
 
-#include <sys/param.h>
-
 #include "virgl_video.h"
 #include "virgl_video_hw.h"
 
@@ -156,7 +154,7 @@ static int sync_dmabuf_to_video_buffer(struct vrend_video_buffer *buf,
                                        const struct virgl_video_dma_buf *dmabuf)
 {
     if (!(dmabuf->flags & VIRGL_VIDEO_DMABUF_READ_ONLY)) {
-        vrend_printf("%s: dmabuf is not readable\n", __func__);
+        virgl_error("%s: dmabuf is not readable\n", __func__);
         return -1;
     }
 
@@ -166,7 +164,7 @@ static int sync_dmabuf_to_video_buffer(struct vrend_video_buffer *buf,
 
         res = vrend_renderer_ctx_res_lookup(buf->ctx->ctx, plane->res_handle);
         if (!res) {
-            vrend_printf("%s: res %d not found\n", __func__, plane->res_handle);
+            virgl_error("%s: res %d not found\n", __func__, plane->res_handle);
             continue;
         }
 
@@ -187,7 +185,7 @@ static int sync_dmabuf_to_video_buffer(struct vrend_video_buffer *buf,
         }
 
         if (EGL_NO_IMAGE_KHR == plane->egl_image) {
-            vrend_printf("%s: create egl image failed\n", __func__);
+            virgl_error("%s: create egl image failed\n", __func__);
             continue;
         }
 
@@ -217,7 +215,7 @@ static int sync_video_buffer_to_dmabuf(struct vrend_video_buffer *buf,
                                        const struct virgl_video_dma_buf *dmabuf)
 {
     if (!(dmabuf->flags & VIRGL_VIDEO_DMABUF_WRITE_ONLY)) {
-        vrend_printf("%s: dmabuf is not writable\n", __func__);
+        virgl_error("%s: dmabuf is not writable\n", __func__);
         return -1;
     }
 
@@ -227,7 +225,7 @@ static int sync_video_buffer_to_dmabuf(struct vrend_video_buffer *buf,
 
         res = vrend_renderer_ctx_res_lookup(buf->ctx->ctx, plane->res_handle);
         if (!res) {
-            vrend_printf("%s: res %d not found\n", __func__, plane->res_handle);
+            virgl_error("%s: res %d not found\n", __func__, plane->res_handle);
             continue;
         }
 
@@ -248,7 +246,7 @@ static int sync_video_buffer_to_dmabuf(struct vrend_video_buffer *buf,
         }
 
         if (EGL_NO_IMAGE_KHR == plane->egl_image) {
-            vrend_printf("%s: create egl image failed\n", __func__);
+            virgl_error("%s: create egl image failed\n", __func__);
             continue;
         }
 
@@ -327,7 +325,7 @@ static void vrend_video_encode_completed(
                                cdc->dest_res->base.width0, GL_MAP_WRITE_BIT);
         for (i = 0, data_size = 0; i < num_coded_bufs &&
                     data_size < cdc->dest_res->base.width0; i++) {
-            size = MIN(cdc->dest_res->base.width0 - data_size, coded_sizes[i]);
+            size = MIN2(cdc->dest_res->base.width0 - data_size, coded_sizes[i]);
             memcpy((uint8_t *)buf + data_size, coded_bufs[i], size);
             vrend_write_to_iovec(cdc->dest_res->iov, cdc->dest_res->num_iovs,
                                  data_size, coded_bufs[i], size);
@@ -338,7 +336,7 @@ static void vrend_video_encode_completed(
         feedback.stat = VIRGL_VIDEO_ENCODE_STAT_SUCCESS;
         feedback.coded_size = data_size;
     } else {
-        vrend_printf("unexcepted coded res type\n");
+        virgl_warn("unexcepted coded res type\n");
         feedback.stat = VIRGL_VIDEO_ENCODE_STAT_FAILURE;
         feedback.coded_size = 0;
     }
@@ -346,7 +344,7 @@ static void vrend_video_encode_completed(
     /* send feedback */
     vrend_write_to_iovec(cdc->feed_res->iov, cdc->feed_res->num_iovs,
                          0, (char *)(&feedback),
-                         MIN(cdc->feed_res->base.width0, sizeof(feedback)));
+                         MIN2(cdc->feed_res->base.width0, sizeof(feedback)));
 
     cdc->dest_res = NULL;
     cdc->feed_res = NULL;
@@ -466,8 +464,10 @@ int vrend_video_create_buffer(struct vrend_video_context *ctx,
     if (buf)
         return 0;
 
-    if (format <= PIPE_FORMAT_NONE || format >= PIPE_FORMAT_COUNT)
+    if (format <= PIPE_FORMAT_NONE || format >= PIPE_FORMAT_COUNT){
+        virgl_error("Invalid vrend video buffer format: %d\n", format);
         return -1;
+    }
 
     if (!width || !height || !res_handles || !num_res)
         return -1;
@@ -618,6 +618,79 @@ static void modify_h265_picture_desc(struct vrend_video_codec *cdc,
     }
 }
 
+static void modify_mpeg12_picture_desc(struct vrend_video_codec *cdc,
+                                       struct vrend_video_buffer *tgt,
+                                       struct virgl_mpeg12_picture_desc *desc)
+{
+    unsigned i;
+    struct vrend_video_buffer *vbuf;
+
+    (void)tgt;
+
+    for (i = 0; i < ARRAY_SIZE(desc->ref); i++) {
+        vbuf = get_video_buffer(cdc->ctx, desc->ref[i]);
+        desc->ref[i] = virgl_video_buffer_id(vbuf ? vbuf->buffer : NULL);
+    }
+}
+
+
+static void modify_mjpeg_picture_desc(struct vrend_video_codec *cdc,
+                                      struct vrend_video_buffer *tgt,
+                                      struct virgl_mjpeg_picture_desc *desc)
+{
+    (void)cdc;
+    (void)tgt;
+    (void)desc;
+}
+
+static void modify_vc1_picture_desc(struct vrend_video_codec *cdc,
+                                    struct vrend_video_buffer *tgt,
+                                    struct virgl_vc1_picture_desc *desc)
+{
+    unsigned i;
+    struct vrend_video_buffer *vbuf;
+
+    (void)tgt;
+
+    for (i = 0; i < ARRAY_SIZE(desc->ref); i++) {
+        vbuf = get_video_buffer(cdc->ctx, desc->ref[i]);
+        desc->ref[i] = virgl_video_buffer_id(vbuf ? vbuf->buffer : NULL);
+    }
+}
+
+static void modify_vp9_picture_desc(struct vrend_video_codec *cdc,
+                                     struct vrend_video_buffer *tgt,
+                                     struct virgl_vp9_picture_desc *desc)
+{
+    unsigned i;
+    struct vrend_video_buffer *vbuf;
+
+    (void)tgt;
+
+    for (i = 0; i < ARRAY_SIZE(desc->ref); i++) {
+        vbuf = get_video_buffer(cdc->ctx, desc->ref[i]);
+        desc->ref[i] = virgl_video_buffer_id(vbuf ? vbuf->buffer : NULL);
+    }
+}
+
+static void modify_av1_picture_desc(struct vrend_video_codec *cdc,
+                                    struct vrend_video_buffer *tgt,
+                                    struct virgl_av1_picture_desc *desc)
+{
+    unsigned i;
+    struct vrend_video_buffer *vbuf;
+
+    (void)tgt;
+
+    for (i = 0; i < ARRAY_SIZE(desc->ref); i++) {
+        vbuf = get_video_buffer(cdc->ctx, desc->ref[i]);
+        desc->ref[i] = virgl_video_buffer_id(vbuf ? vbuf->buffer : NULL);
+    }
+
+    vbuf = get_video_buffer(cdc->ctx, desc->film_grain_target);
+    desc->film_grain_target = virgl_video_buffer_id(vbuf ? vbuf->buffer : NULL);
+}
+
 static void modify_picture_desc(struct vrend_video_codec *cdc,
                                 struct vrend_video_buffer *tgt,
                                 union virgl_picture_desc *desc)
@@ -640,6 +713,25 @@ static void modify_picture_desc(struct vrend_video_codec *cdc,
     case PIPE_VIDEO_PROFILE_HEVC_MAIN_444:
         modify_h265_picture_desc(cdc, tgt, &desc->h265);
         break;
+    case PIPE_VIDEO_PROFILE_MPEG2_MAIN:
+    case PIPE_VIDEO_PROFILE_MPEG2_SIMPLE:
+        modify_mpeg12_picture_desc(cdc, tgt, &desc->mpeg12);
+        break;
+    case PIPE_VIDEO_PROFILE_JPEG_BASELINE:
+        modify_mjpeg_picture_desc(cdc, tgt, &desc->mjpeg);
+        break;
+    case PIPE_VIDEO_PROFILE_VC1_SIMPLE:
+    case PIPE_VIDEO_PROFILE_VC1_MAIN:
+    case PIPE_VIDEO_PROFILE_VC1_ADVANCED:
+        modify_vc1_picture_desc(cdc, tgt, &desc->vc1);
+        break;
+    case PIPE_VIDEO_PROFILE_VP9_PROFILE0:
+    case PIPE_VIDEO_PROFILE_VP9_PROFILE2:
+        modify_vp9_picture_desc(cdc, tgt, &desc->vp9);
+        break;
+    case PIPE_VIDEO_PROFILE_AV1_MAIN:
+        modify_av1_picture_desc(cdc, tgt, &desc->av1);
+        break;
     default:
         break;
     }
@@ -661,26 +753,28 @@ int vrend_video_decode_bitstream(struct vrend_video_context *ctx,
     struct vrend_video_buffer *tgt = get_video_buffer(ctx, tgt_handle);
     union virgl_picture_desc desc;
 
-    if (!cdc || !tgt)
+    if (!cdc || !tgt){
+        virgl_error("video codec: %p, video buffer: %p, invalid.\n", (void *)cdc, (void *)tgt);
         return -1;
+    }
 
     bs_buffers = calloc(num_buffers, sizeof(void *));
     if (!bs_buffers) {
-        vrend_printf("%s: alloc bs_buffers failed\n", __func__);
+        virgl_error("%s: alloc bs_buffers failed\n", __func__);
         return -1;
     }
 
     bs_sizes = calloc(num_buffers, sizeof(unsigned));
     if (!bs_sizes) {
-        vrend_printf("%s: alloc bs_sizes failed\n", __func__);
+        virgl_error("%s: alloc bs_sizes failed\n", __func__);
         goto err;
     }
 
     for (i = 0, num_bs = 0; i < num_buffers; i++) {
         res = vrend_renderer_ctx_res_lookup(ctx->ctx, buffer_handles[i]);
         if (!res || !res->ptr) {
-            vrend_printf("%s: bs res %d invalid or not found",
-                         __func__, buffer_handles[i]);
+            virgl_warn("%s: bs res %d invalid or not found",
+                       __func__, buffer_handles[i]);
             continue;
         }
 
@@ -693,12 +787,12 @@ int vrend_video_decode_bitstream(struct vrend_video_context *ctx,
 
     res = vrend_renderer_ctx_res_lookup(ctx->ctx, desc_handle);
     if (!res) {
-        vrend_printf("%s: desc res %d not found\n", __func__, desc_handle);
+        virgl_error("%s: desc res %d not found\n", __func__, desc_handle);
         goto err;
     }
     memset(&desc, 0, sizeof(desc));
     vrend_read_from_iovec(res->iov, res->num_iovs, 0, (char *)(&desc),
-                          MIN(res->base.width0, sizeof(desc)));
+                          MIN2(res->base.width0, sizeof(desc)));
     modify_picture_desc(cdc, tgt, &desc);
 
     err = virgl_video_decode_bitstream(cdc->codec, tgt->buffer, &desc,
@@ -729,24 +823,24 @@ int vrend_video_encode_bitstream(struct vrend_video_context *ctx,
     /* Feedback resource */
     feed_res = vrend_renderer_ctx_res_lookup(ctx->ctx, feed_handle);
     if (!feed_res) {
-        vrend_printf("%s: feedback res %d not found\n", __func__, feed_handle);
+        virgl_error("%s: feedback res %d not found\n", __func__, feed_handle);
         return -1;
     }
 
     /* Picture descriptor resource */
     desc_res = vrend_renderer_ctx_res_lookup(ctx->ctx, desc_handle);
     if (!desc_res) {
-        vrend_printf("%s: desc res %d not found\n", __func__, desc_handle);
+        virgl_error("%s: desc res %d not found\n", __func__, desc_handle);
         return -1;
     }
     memset(&desc, 0, sizeof(desc));
     vrend_read_from_iovec(desc_res->iov, desc_res->num_iovs, 0, (char *)(&desc),
-                          MIN(desc_res->base.width0, sizeof(desc)));
+                          MIN2(desc_res->base.width0, sizeof(desc)));
 
     /* Destination buffer resource. */
     dest_res = vrend_renderer_ctx_res_lookup(ctx->ctx, dest_handle);
     if (!dest_res) {
-        vrend_printf("%s: dest res %d not found\n", __func__, dest_handle);
+        virgl_error("%s: dest res %d not found\n", __func__, dest_handle);
         return -1;
     }
 
