@@ -25,9 +25,9 @@
 #include <stdlib.h>
 #include <time.h>
 #include <errno.h>
-#include <sys/uio.h>
 #include <virglrenderer.h>
 #include "virgl_hw.h"
+#include "vrend_iov.h"
 #include "pipe/p_format.h"
 #include "testvirgl_encode.h"
 #include "virgl_protocol.h"
@@ -45,23 +45,23 @@ START_TEST(virgl_test_overlap_obj_id)
 
     /* set blend state */
     {
-	struct pipe_blend_state blend;
-	int blend_handle = ctx_handle;
-	memset(&blend, 0, sizeof(blend));
-	blend.rt[0].colormask = PIPE_MASK_RGBA;
-	virgl_encode_blend_state(&ctx, blend_handle, &blend);
-	virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
+        struct pipe_blend_state blend;
+        int blend_handle = ctx_handle;
+        memset(&blend, 0, sizeof(blend));
+        blend.rt[0].colormask = PIPE_MASK_RGBA;
+        virgl_encode_blend_state(&ctx, blend_handle, &blend);
+        virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
     }
 
     /* set depth stencil alpha state */
     {
-	struct pipe_depth_stencil_alpha_state dsa;
-	int dsa_handle = ctx_handle;
-	memset(&dsa, 0, sizeof(dsa));
-	dsa.depth.writemask = 1;
-	dsa.depth.func = PIPE_FUNC_LESS;
-	virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
-	virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
+        struct pipe_depth_stencil_alpha_state dsa;
+        int dsa_handle = ctx_handle;
+        memset(&dsa, 0, sizeof(dsa));
+        dsa.depth.writemask = 1;
+        dsa.depth.func = PIPE_FUNC_LESS;
+        virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
+        virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
     }
     testvirgl_fini_ctx_cmdbuf(&ctx);
 }
@@ -118,7 +118,7 @@ START_TEST(virgl_test_clear)
     virgl_encode_clear(&ctx, PIPE_CLEAR_COLOR0, &color, 0.0, 0);
 
     /* submit the cmd stream */
-    virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+    testvirgl_ctx_send_cmdbuf(&ctx);
 
     /* read back the cleared values in the resource */
     box.x = 0;
@@ -132,8 +132,8 @@ START_TEST(virgl_test_clear)
 
     /* check the returned values */
     for (i = 0; i < 5; i++) {
-	uint32_t *ptr = res.iovs[0].iov_base;
-	ck_assert_int_eq(ptr[i], test_green);
+        uint32_t *ptr = res.iovs[0].iov_base;
+        ck_assert_int_eq(ptr[i], test_green);
     }
 
     /* cleanup */
@@ -207,7 +207,7 @@ START_TEST(virgl_test_blit_simple)
     virgl_encode_blit(&ctx, &res2, &res, &blit);
 
     /* submit the cmd stream */
-    virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+    testvirgl_ctx_send_cmdbuf(&ctx);
 
     /* read back the cleared values in the resource */
     box.x = 0;
@@ -221,8 +221,8 @@ START_TEST(virgl_test_blit_simple)
 
     /* check the returned values */
     for (i = 0; i < 5; i++) {
-	uint32_t *ptr = res2.iovs[0].iov_base;
-	ck_assert_int_eq(ptr[i], test_green);
+        uint32_t *ptr = res2.iovs[0].iov_base;
+        ck_assert_int_eq(ptr[i], test_green);
     }
 
     /* cleanup */
@@ -338,36 +338,36 @@ START_TEST(virgl_test_render_simple)
 
     /* create vertex shader */
     {
-	struct pipe_shader_state vs;
+        struct pipe_shader_state vs;
          const char *text =
-	   "VERT\n"
-	   "DCL IN[0]\n"
-	   "DCL IN[1]\n"
-	   "DCL OUT[0], POSITION\n"
-	   "DCL OUT[1], COLOR\n"
-	   "  0: MOV OUT[1], IN[1]\n"
-	   "  1: MOV OUT[0], IN[0]\n"
-	   "  2: END\n";
-	 memset(&vs, 0, sizeof(vs));
-	 vs_handle = ctx_handle++;
-	 virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
-				   &vs, text);
-	 virgl_encode_bind_shader(&ctx, vs_handle, PIPE_SHADER_VERTEX);
+           "VERT\n"
+           "DCL IN[0]\n"
+           "DCL IN[1]\n"
+           "DCL OUT[0], POSITION\n"
+           "DCL OUT[1], COLOR\n"
+           "  0: MOV OUT[1], IN[1]\n"
+           "  1: MOV OUT[0], IN[0]\n"
+           "  2: END\n";
+         memset(&vs, 0, sizeof(vs));
+         vs_handle = ctx_handle++;
+         virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
+                                   &vs, text);
+         virgl_encode_bind_shader(&ctx, vs_handle, PIPE_SHADER_VERTEX);
     }
 
     /* create fragment shader */
     {
-	struct pipe_shader_state fs;
-	const char *text =
-	    "FRAG\n"
-	    "DCL IN[0], COLOR, LINEAR\n"
-	    "DCL OUT[0], COLOR\n"
-	    "  0: MOV OUT[0], IN[0]\n"
-	    "  1: END\n";
-	memset(&fs, 0, sizeof(fs));
-	fs_handle = ctx_handle++;
-	virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
-				   &fs, text);
+        struct pipe_shader_state fs;
+        const char *text =
+            "FRAG\n"
+            "DCL IN[0], COLOR, LINEAR\n"
+            "DCL OUT[0], COLOR\n"
+            "  0: MOV OUT[0], IN[0]\n"
+            "  1: END\n";
+        memset(&fs, 0, sizeof(fs));
+        fs_handle = ctx_handle++;
+        virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
+                                   &fs, text);
 
         virgl_encode_bind_shader(&ctx, fs_handle, PIPE_SHADER_FRAGMENT);
     }
@@ -383,66 +383,66 @@ START_TEST(virgl_test_render_simple)
 
     /* set blend state */
     {
-	struct pipe_blend_state blend;
-	int blend_handle = ctx_handle++;
-	memset(&blend, 0, sizeof(blend));
-	blend.rt[0].colormask = PIPE_MASK_RGBA;
-	virgl_encode_blend_state(&ctx, blend_handle, &blend);
-	virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
+        struct pipe_blend_state blend;
+        int blend_handle = ctx_handle++;
+        memset(&blend, 0, sizeof(blend));
+        blend.rt[0].colormask = PIPE_MASK_RGBA;
+        virgl_encode_blend_state(&ctx, blend_handle, &blend);
+        virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
     }
 
     /* set depth stencil alpha state */
     {
-	struct pipe_depth_stencil_alpha_state dsa;
-	int dsa_handle = ctx_handle++;
-	memset(&dsa, 0, sizeof(dsa));
-	dsa.depth.writemask = 1;
-	dsa.depth.func = PIPE_FUNC_LESS;
-	virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
-	virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
+        struct pipe_depth_stencil_alpha_state dsa;
+        int dsa_handle = ctx_handle++;
+        memset(&dsa, 0, sizeof(dsa));
+        dsa.depth.writemask = 1;
+        dsa.depth.func = PIPE_FUNC_LESS;
+        virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
+        virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
     }
 
     /* set rasterizer state */
     {
-	struct pipe_rasterizer_state rasterizer;
-	int rs_handle = ctx_handle++;
-	memset(&rasterizer, 0, sizeof(rasterizer));
-	rasterizer.cull_face = PIPE_FACE_NONE;
-	rasterizer.half_pixel_center = 1;
-	rasterizer.bottom_edge_rule = 1;
-	rasterizer.depth_clip = 1;
-	virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
-	virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
+        struct pipe_rasterizer_state rasterizer;
+        int rs_handle = ctx_handle++;
+        memset(&rasterizer, 0, sizeof(rasterizer));
+        rasterizer.cull_face = PIPE_FACE_NONE;
+        rasterizer.half_pixel_center = 1;
+        rasterizer.bottom_edge_rule = 1;
+        rasterizer.depth_clip = 1;
+        virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
+        virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
     }
 
     /* set viewport state */
     {
-	struct pipe_viewport_state vp;
-	float znear = 0, zfar = 1.0;
-	float half_w = tw / 2.0f;
-	float half_h = th / 2.0f;
-	float half_d = (zfar - znear) / 2.0f;
+        struct pipe_viewport_state vp;
+        float znear = 0, zfar = 1.0;
+        float half_w = tw / 2.0f;
+        float half_h = th / 2.0f;
+        float half_d = (zfar - znear) / 2.0f;
 
-	vp.scale[0] = half_w;
-	vp.scale[1] = half_h;
-	vp.scale[2] = half_d;
+        vp.scale[0] = half_w;
+        vp.scale[1] = half_h;
+        vp.scale[2] = half_d;
 
-	vp.translate[0] = half_w + 0;
-	vp.translate[1] = half_h + 0;
-	vp.translate[2] = half_d + znear;
-	virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
+        vp.translate[0] = half_w + 0;
+        vp.translate[1] = half_h + 0;
+        vp.translate[2] = half_d + znear;
+        virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
     }
 
     /* draw */
     {
-	struct pipe_draw_info info;
-	memset(&info, 0, sizeof(info));
-	info.count = 3;
-	info.mode = PIPE_PRIM_TRIANGLES;
-	virgl_encoder_draw_vbo(&ctx, &info);
+        struct pipe_draw_info info;
+        memset(&info, 0, sizeof(info));
+        info.count = 3;
+        info.mode = PIPE_PRIM_TRIANGLES;
+        virgl_encoder_draw_vbo(&ctx, &info);
     }
 
-    virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+    testvirgl_ctx_send_cmdbuf(&ctx);
 
     /* create a fence */
     testvirgl_reset_fence();
@@ -450,13 +450,13 @@ START_TEST(virgl_test_render_simple)
     ck_assert_int_eq(ret, 0);
 
     do {
-	int fence;
+        int fence;
 
-	virgl_renderer_poll();
-	fence = testvirgl_get_last_fence();
-	if (fence >= 1)
-	    break;
-	nanosleep((struct timespec[]){{0, 50000}}, NULL);
+        virgl_renderer_poll();
+        fence = testvirgl_get_last_fence();
+        if (fence >= 1)
+            break;
+        nanosleep((struct timespec[]){{0, 50000}}, NULL);
     } while(1);
 
     /* read back the tri values in the resource */
@@ -470,16 +470,16 @@ START_TEST(virgl_test_render_simple)
     ck_assert_int_eq(ret, 0);
 
     {
-	int w, h;
-	bool all_cleared = true;
-	uint32_t *ptr = res.iovs[0].iov_base;
-	for (h = 0; h < th; h++) {
-	    for (w = 0; w < tw; w++) {
-		if (ptr[h * tw + w] != test_green)
-		    all_cleared = false;
-	    }
-	}
-	ck_assert_int_eq(all_cleared, false);
+        int w, h;
+        bool all_cleared = true;
+        uint32_t *ptr = res.iovs[0].iov_base;
+        for (h = 0; h < th; h++) {
+            for (w = 0; w < tw; w++) {
+                if (ptr[h * tw + w] != test_green)
+                    all_cleared = false;
+            }
+        }
+        ck_assert_int_eq(all_cleared, false);
     }
 
     /* cleanup */
@@ -580,133 +580,133 @@ START_TEST(virgl_test_render_geom_simple)
 
     /* create vertex shader */
     {
-	struct pipe_shader_state vs;
+        struct pipe_shader_state vs;
          const char *text =
-	   "VERT\n"
-	   "DCL IN[0]\n"
-	   "DCL IN[1]\n"
-	   "DCL OUT[0], POSITION\n"
-	   "DCL OUT[1], GENERIC[20]\n"
-	   "  0: MOV OUT[1], IN[1]\n"
-	   "  1: MOV OUT[0], IN[0]\n"
-	   "  2: END\n";
-	 memset(&vs, 0, sizeof(vs));
-	 vs_handle = ctx_handle++;
-	 virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
-				   &vs, text);
+           "VERT\n"
+           "DCL IN[0]\n"
+           "DCL IN[1]\n"
+           "DCL OUT[0], POSITION\n"
+           "DCL OUT[1], GENERIC[20]\n"
+           "  0: MOV OUT[1], IN[1]\n"
+           "  1: MOV OUT[0], IN[0]\n"
+           "  2: END\n";
+         memset(&vs, 0, sizeof(vs));
+         vs_handle = ctx_handle++;
+         virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
+                                   &vs, text);
          virgl_encode_bind_shader(&ctx, vs_handle, PIPE_SHADER_VERTEX);
     }
 
     /* create geometry shader */
     {
-	struct pipe_shader_state gs;
+        struct pipe_shader_state gs;
          const char *text =
-	   "GEOM\n"
-	   "PROPERTY GS_INPUT_PRIMITIVE TRIANGLES\n"
-	   "PROPERTY GS_OUTPUT_PRIMITIVE TRIANGLE_STRIP\n"
-	   "PROPERTY GS_MAX_OUTPUT_VERTICES 3\n"
-	   "PROPERTY GS_INVOCATIONS 1\n"
-	   "DCL IN[][0], POSITION\n"
-	   "DCL IN[][1], GENERIC[20]\n"
-	   "DCL OUT[0], POSITION\n"
-	   "DCL OUT[1], GENERIC[20]\n"
-	   "IMM[0] INT32 {0, 0, 0, 0}\n"
-	   "0:MOV OUT[0], IN[0][0]\n"
-	   "1:MOV OUT[1], IN[0][1]\n"
-	   "2:EMIT IMM[0].xxxx\n"
-	   "3:MOV OUT[0], IN[1][0]\n"
-	   "4:MOV OUT[1], IN[0][1]\n" /* copy color from input vertex 0 */
-	   "5:EMIT IMM[0].xxxx\n"
-	   "6:MOV OUT[0], IN[2][0]\n"
-	   "7:MOV OUT[1], IN[2][1]\n"
-	   "8:EMIT IMM[0].xxxx\n"
-	   "9:END\n";
-	 memset(&gs, 0, sizeof(gs));
-	 gs_handle = ctx_handle++;
-	 virgl_encode_shader_state(&ctx, gs_handle, PIPE_SHADER_GEOMETRY,
-				   &gs, text);
+           "GEOM\n"
+           "PROPERTY GS_INPUT_PRIMITIVE TRIANGLES\n"
+           "PROPERTY GS_OUTPUT_PRIMITIVE TRIANGLE_STRIP\n"
+           "PROPERTY GS_MAX_OUTPUT_VERTICES 3\n"
+           "PROPERTY GS_INVOCATIONS 1\n"
+           "DCL IN[][0], POSITION\n"
+           "DCL IN[][1], GENERIC[20]\n"
+           "DCL OUT[0], POSITION\n"
+           "DCL OUT[1], GENERIC[20]\n"
+           "IMM[0] INT32 {0, 0, 0, 0}\n"
+           "0:MOV OUT[0], IN[0][0]\n"
+           "1:MOV OUT[1], IN[0][1]\n"
+           "2:EMIT IMM[0].xxxx\n"
+           "3:MOV OUT[0], IN[1][0]\n"
+           "4:MOV OUT[1], IN[0][1]\n" /* copy color from input vertex 0 */
+           "5:EMIT IMM[0].xxxx\n"
+           "6:MOV OUT[0], IN[2][0]\n"
+           "7:MOV OUT[1], IN[2][1]\n"
+           "8:EMIT IMM[0].xxxx\n"
+           "9:END\n";
+         memset(&gs, 0, sizeof(gs));
+         gs_handle = ctx_handle++;
+         virgl_encode_shader_state(&ctx, gs_handle, PIPE_SHADER_GEOMETRY,
+                                   &gs, text);
          virgl_encode_bind_shader(&ctx, gs_handle, PIPE_SHADER_GEOMETRY);
     }
 
     /* create fragment shader */
     {
-	struct pipe_shader_state fs;
-	const char *text =
-	    "FRAG\n"
-	    "DCL IN[0], GENERIC[20], LINEAR\n"
-	    "DCL OUT[0], COLOR\n"
-	    "  0: MOV OUT[0], IN[0]\n"
-	    "  1: END\n";
-	memset(&fs, 0, sizeof(fs));
-	fs_handle = ctx_handle++;
-	virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
-				   &fs, text);
+        struct pipe_shader_state fs;
+        const char *text =
+            "FRAG\n"
+            "DCL IN[0], GENERIC[20], LINEAR\n"
+            "DCL OUT[0], COLOR\n"
+            "  0: MOV OUT[0], IN[0]\n"
+            "  1: END\n";
+        memset(&fs, 0, sizeof(fs));
+        fs_handle = ctx_handle++;
+        virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
+                                   &fs, text);
 
         virgl_encode_bind_shader(&ctx, fs_handle, PIPE_SHADER_FRAGMENT);
     }
 
     /* set blend state */
     {
-	struct pipe_blend_state blend;
-	int blend_handle = ctx_handle++;
-	memset(&blend, 0, sizeof(blend));
-	blend.rt[0].colormask = PIPE_MASK_RGBA;
-	virgl_encode_blend_state(&ctx, blend_handle, &blend);
-	virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
+        struct pipe_blend_state blend;
+        int blend_handle = ctx_handle++;
+        memset(&blend, 0, sizeof(blend));
+        blend.rt[0].colormask = PIPE_MASK_RGBA;
+        virgl_encode_blend_state(&ctx, blend_handle, &blend);
+        virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
     }
 
     /* set depth stencil alpha state */
     {
-	struct pipe_depth_stencil_alpha_state dsa;
-	int dsa_handle = ctx_handle++;
-	memset(&dsa, 0, sizeof(dsa));
-	dsa.depth.writemask = 1;
-	dsa.depth.func = PIPE_FUNC_LESS;
-	virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
-	virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
+        struct pipe_depth_stencil_alpha_state dsa;
+        int dsa_handle = ctx_handle++;
+        memset(&dsa, 0, sizeof(dsa));
+        dsa.depth.writemask = 1;
+        dsa.depth.func = PIPE_FUNC_LESS;
+        virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
+        virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
     }
 
     /* set rasterizer state */
     {
-	struct pipe_rasterizer_state rasterizer;
-	int rs_handle = ctx_handle++;
-	memset(&rasterizer, 0, sizeof(rasterizer));
-	rasterizer.cull_face = PIPE_FACE_NONE;
-	rasterizer.half_pixel_center = 1;
-	rasterizer.bottom_edge_rule = 1;
-	rasterizer.depth_clip = 1;
-	virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
-	virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
+        struct pipe_rasterizer_state rasterizer;
+        int rs_handle = ctx_handle++;
+        memset(&rasterizer, 0, sizeof(rasterizer));
+        rasterizer.cull_face = PIPE_FACE_NONE;
+        rasterizer.half_pixel_center = 1;
+        rasterizer.bottom_edge_rule = 1;
+        rasterizer.depth_clip = 1;
+        virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
+        virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
     }
 
     /* set viewport state */
     {
-	struct pipe_viewport_state vp;
-	float znear = 0, zfar = 1.0;
-	float half_w = tw / 2.0f;
-	float half_h = th / 2.0f;
-	float half_d = (zfar - znear) / 2.0f;
+        struct pipe_viewport_state vp;
+        float znear = 0, zfar = 1.0;
+        float half_w = tw / 2.0f;
+        float half_h = th / 2.0f;
+        float half_d = (zfar - znear) / 2.0f;
 
-	vp.scale[0] = half_w;
-	vp.scale[1] = half_h;
-	vp.scale[2] = half_d;
+        vp.scale[0] = half_w;
+        vp.scale[1] = half_h;
+        vp.scale[2] = half_d;
 
-	vp.translate[0] = half_w + 0;
-	vp.translate[1] = half_h + 0;
-	vp.translate[2] = half_d + znear;
-	virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
+        vp.translate[0] = half_w + 0;
+        vp.translate[1] = half_h + 0;
+        vp.translate[2] = half_d + znear;
+        virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
     }
 
     /* draw */
     {
-	struct pipe_draw_info info;
-	memset(&info, 0, sizeof(info));
-	info.count = 3;
-	info.mode = PIPE_PRIM_TRIANGLES;
-	virgl_encoder_draw_vbo(&ctx, &info);
+        struct pipe_draw_info info;
+        memset(&info, 0, sizeof(info));
+        info.count = 3;
+        info.mode = PIPE_PRIM_TRIANGLES;
+        virgl_encoder_draw_vbo(&ctx, &info);
     }
 
-    virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+    testvirgl_ctx_send_cmdbuf(&ctx);
 
     /* create a fence */
     testvirgl_reset_fence();
@@ -714,13 +714,13 @@ START_TEST(virgl_test_render_geom_simple)
     ck_assert_int_eq(ret, 0);
 
     do {
-	int fence;
+        int fence;
 
-	virgl_renderer_poll();
-	fence = testvirgl_get_last_fence();
-	if (fence >= 1)
-	    break;
-	nanosleep((struct timespec[]){{0, 50000}}, NULL);
+        virgl_renderer_poll();
+        fence = testvirgl_get_last_fence();
+        if (fence >= 1)
+            break;
+        nanosleep((struct timespec[]){{0, 50000}}, NULL);
     } while(1);
 
     /* read back the tri values in the resource */
@@ -734,16 +734,16 @@ START_TEST(virgl_test_render_geom_simple)
     ck_assert_int_eq(ret, 0);
 
     {
-	int w, h;
-	bool all_cleared = true;
-	uint32_t *ptr = res.iovs[0].iov_base;
-	for (h = 0; h < th; h++) {
-	    for (w = 0; w < tw; w++) {
-		if (ptr[h * tw + w] != test_green)
-		    all_cleared = false;
-	    }
-	}
-	ck_assert_int_eq(all_cleared, false);
+        int w, h;
+        bool all_cleared = true;
+        uint32_t *ptr = res.iovs[0].iov_base;
+        for (h = 0; h < th; h++) {
+            for (w = 0; w < tw; w++) {
+                if (ptr[h * tw + w] != test_green)
+                    all_cleared = false;
+            }
+        }
+        ck_assert_int_eq(all_cleared, false);
     }
 
     /* cleanup */
@@ -854,104 +854,104 @@ START_TEST(virgl_test_render_xfb)
 
     /* create vertex shader */
     {
-	struct pipe_shader_state vs;
+        struct pipe_shader_state vs;
          const char *text =
-	   "VERT\n"
-	   "DCL IN[0]\n"
-	   "DCL IN[1]\n"
-	   "DCL OUT[0], POSITION\n"
-	   "DCL OUT[1], COLOR\n"
-	   "  0: MOV OUT[1], IN[1]\n"
-	   "  1: MOV OUT[0], IN[0]\n"
-	   "  2: END\n";
-	 memset(&vs, 0, sizeof(vs));
-	 vs_handle = ctx_handle++;
-	 vs.stream_output.num_outputs = 1;
-	 vs.stream_output.stride[0] = 4;
-	 vs.stream_output.output[0].num_components = 4;
-	 virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
-				   &vs, text);
+           "VERT\n"
+           "DCL IN[0]\n"
+           "DCL IN[1]\n"
+           "DCL OUT[0], POSITION\n"
+           "DCL OUT[1], COLOR\n"
+           "  0: MOV OUT[1], IN[1]\n"
+           "  1: MOV OUT[0], IN[0]\n"
+           "  2: END\n";
+         memset(&vs, 0, sizeof(vs));
+         vs_handle = ctx_handle++;
+         vs.stream_output.num_outputs = 1;
+         vs.stream_output.stride[0] = 4;
+         vs.stream_output.output[0].num_components = 4;
+         virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
+                                   &vs, text);
          virgl_encode_bind_shader(&ctx, vs_handle, PIPE_SHADER_VERTEX);
     }
 
     /* create fragment shader */
     {
-	struct pipe_shader_state fs;
-	const char *text =
-	    "FRAG\n"
-	    "DCL IN[0], COLOR, LINEAR\n"
-	    "DCL OUT[0], COLOR\n"
-	    "  0: MOV OUT[0], IN[0]\n"
-	    "  1: END\n";
-	memset(&fs, 0, sizeof(fs));
-	fs_handle = ctx_handle++;
-	virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
-				   &fs, text);
+        struct pipe_shader_state fs;
+        const char *text =
+            "FRAG\n"
+            "DCL IN[0], COLOR, LINEAR\n"
+            "DCL OUT[0], COLOR\n"
+            "  0: MOV OUT[0], IN[0]\n"
+            "  1: END\n";
+        memset(&fs, 0, sizeof(fs));
+        fs_handle = ctx_handle++;
+        virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
+                                   &fs, text);
         virgl_encode_bind_shader(&ctx, fs_handle, PIPE_SHADER_FRAGMENT);
     }
 
     /* set blend state */
     {
-	struct pipe_blend_state blend;
-	int blend_handle = ctx_handle++;
-	memset(&blend, 0, sizeof(blend));
-	blend.rt[0].colormask = PIPE_MASK_RGBA;
-	virgl_encode_blend_state(&ctx, blend_handle, &blend);
-	virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
+        struct pipe_blend_state blend;
+        int blend_handle = ctx_handle++;
+        memset(&blend, 0, sizeof(blend));
+        blend.rt[0].colormask = PIPE_MASK_RGBA;
+        virgl_encode_blend_state(&ctx, blend_handle, &blend);
+        virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
     }
 
     /* set depth stencil alpha state */
     {
-	struct pipe_depth_stencil_alpha_state dsa;
-	int dsa_handle = ctx_handle++;
-	memset(&dsa, 0, sizeof(dsa));
-	dsa.depth.writemask = 1;
-	dsa.depth.func = PIPE_FUNC_LESS;
-	virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
-	virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
+        struct pipe_depth_stencil_alpha_state dsa;
+        int dsa_handle = ctx_handle++;
+        memset(&dsa, 0, sizeof(dsa));
+        dsa.depth.writemask = 1;
+        dsa.depth.func = PIPE_FUNC_LESS;
+        virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
+        virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
     }
 
     /* set rasterizer state */
     {
-	struct pipe_rasterizer_state rasterizer;
-	int rs_handle = ctx_handle++;
-	memset(&rasterizer, 0, sizeof(rasterizer));
-	rasterizer.cull_face = PIPE_FACE_NONE;
-	rasterizer.half_pixel_center = 1;
-	rasterizer.bottom_edge_rule = 1;
-	rasterizer.depth_clip = 1;
-	virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
-	virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
+        struct pipe_rasterizer_state rasterizer;
+        int rs_handle = ctx_handle++;
+        memset(&rasterizer, 0, sizeof(rasterizer));
+        rasterizer.cull_face = PIPE_FACE_NONE;
+        rasterizer.half_pixel_center = 1;
+        rasterizer.bottom_edge_rule = 1;
+        rasterizer.depth_clip = 1;
+        virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
+        virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
     }
 
     /* set viewport state */
     {
-	struct pipe_viewport_state vp;
-	float znear = 0, zfar = 1.0;
-	float half_w = tw / 2.0f;
-	float half_h = th / 2.0f;
-	float half_d = (zfar - znear) / 2.0f;
+        struct pipe_viewport_state vp;
+        float znear = 0, zfar = 1.0;
+        float half_w = tw / 2.0f;
+        float half_h = th / 2.0f;
+        float half_d = (zfar - znear) / 2.0f;
 
-	vp.scale[0] = half_w;
-	vp.scale[1] = half_h;
-	vp.scale[2] = half_d;
+        vp.scale[0] = half_w;
+        vp.scale[1] = half_h;
+        vp.scale[2] = half_d;
 
-	vp.translate[0] = half_w + 0;
-	vp.translate[1] = half_h + 0;
-	vp.translate[2] = half_d + znear;
-	virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
+        vp.translate[0] = half_w + 0;
+        vp.translate[1] = half_h + 0;
+        vp.translate[2] = half_d + znear;
+        virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
     }
 
     /* draw */
     {
-	struct pipe_draw_info info;
-	memset(&info, 0, sizeof(info));
-	info.count = 3;
-	info.mode = PIPE_PRIM_TRIANGLES;
-	virgl_encoder_draw_vbo(&ctx, &info);
+        struct pipe_draw_info info;
+        memset(&info, 0, sizeof(info));
+        info.count = 3;
+        info.mode = PIPE_PRIM_TRIANGLES;
+        virgl_encoder_draw_vbo(&ctx, &info);
     }
 
-    virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+    testvirgl_ctx_send_cmdbuf(&ctx);
 
     /* create a fence */
     testvirgl_reset_fence();
@@ -959,13 +959,13 @@ START_TEST(virgl_test_render_xfb)
     ck_assert_int_eq(ret, 0);
 
     do {
-	int fence;
+        int fence;
 
-	virgl_renderer_poll();
-	fence = testvirgl_get_last_fence();
-	if (fence >= 1)
-	    break;
-	nanosleep((struct timespec[]){{0, 50000}}, NULL);
+        virgl_renderer_poll();
+        fence = testvirgl_get_last_fence();
+        if (fence >= 1)
+            break;
+        nanosleep((struct timespec[]){{0, 50000}}, NULL);
     } while(1);
 
     /* read back the tri values in the resource */
@@ -979,16 +979,16 @@ START_TEST(virgl_test_render_xfb)
     ck_assert_int_eq(ret, 0);
 
     {
-	int w, h;
-	bool all_cleared = true;
-	uint32_t *ptr = res.iovs[0].iov_base;
-	for (h = 0; h < th; h++) {
-	    for (w = 0; w < tw; w++) {
-		if (ptr[h * tw + w] != test_green)
-		    all_cleared = false;
-	    }
-	}
-	ck_assert_int_eq(all_cleared, false);
+        int w, h;
+        bool all_cleared = true;
+        uint32_t *ptr = res.iovs[0].iov_base;
+        for (h = 0; h < th; h++) {
+            for (w = 0; w < tw; w++) {
+                if (ptr[h * tw + w] != test_green)
+                    all_cleared = false;
+            }
+        }
+        ck_assert_int_eq(all_cleared, false);
     }
 
     /* cleanup */
@@ -1029,6 +1029,285 @@ START_TEST(virgl_test_large_shader)
 }
 END_TEST
 
+START_TEST(virgl_test_set_viewport_state)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_viewport_state vp[PIPE_MAX_VIEWPORTS];
+   int start_slot = 0;
+   int viewports_count = 1;
+
+   /* Filling the struct with real values is actually not needed, clearing to 0 should suffice if we
+       don't want to re-use the context  for more drawing */
+   int tw = 300, th = 300;
+
+   float znear = 0, zfar = 1.0;
+   float half_w = tw / 2.0f;
+   float half_h = th / 2.0f;
+   float half_d = (zfar - znear) / 2.0f;
+
+   vp[0].scale[0] = half_w;
+   vp[0].scale[1] = half_h;
+   vp[0].scale[2] = half_d;
+
+   vp[0].translate[0] = half_w + 0;
+   vp[0].translate[1] = half_h + 0;
+   vp[0].translate[2] = half_d + znear;
+
+   /* Here we are in range, no error expected */
+   ret = virgl_encoder_set_viewport_states(&ctx, start_slot, viewports_count, vp);
+   ck_assert_int_eq(ret, 0);
+
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Too many viewports */
+   viewports_count = PIPE_MAX_VIEWPORTS + 1;
+
+   ret = virgl_encoder_set_viewport_states(&ctx, start_slot, viewports_count, vp);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Using too large index to place viewport */
+   start_slot = PIPE_MAX_VIEWPORTS;
+   viewports_count = 1;
+
+   ret = virgl_encoder_set_viewport_states(&ctx, start_slot, viewports_count, vp);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Combination of previous conditions */
+   start_slot = 2;
+   viewports_count = PIPE_MAX_VIEWPORTS + 1;
+
+   ret = virgl_encoder_set_viewport_states(&ctx, start_slot, viewports_count, vp);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_decode_set_scissor_state)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_scissor_state ss[PIPE_MAX_VIEWPORTS + 1] = {0};
+
+   int start_slot = 0;
+   int scissors_count = 2;
+
+   /* Here we are using correct values, no error expected */
+   ret = virgl_encoder_set_scissor_state(&ctx, start_slot, scissors_count, ss);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Too many scissors for viewports */
+   start_slot = 0;
+   scissors_count = PIPE_MAX_VIEWPORTS + 1;
+
+   ret = virgl_encoder_set_scissor_state(&ctx, start_slot, scissors_count, ss);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Using too large index to place scissors */
+   start_slot = PIPE_MAX_VIEWPORTS;
+   scissors_count = 2;
+
+   ret = virgl_encoder_set_scissor_state(&ctx, start_slot, scissors_count, ss);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Combination of previous conditions */
+   start_slot = 2;
+   scissors_count = PIPE_MAX_VIEWPORTS + 1 - 2;
+
+   ret = virgl_encoder_set_scissor_state(&ctx, start_slot, scissors_count, ss);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_decode_set_constant_buffer)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   uint8_t constant_buffer[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
+
+   /* Here we are using correct values, no error expected */
+   ret = virgl_encoder_write_constant_buffer(&ctx, PIPE_SHADER_VERTEX, 1, 10, constant_buffer);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Sending a NULL constant buffer */
+   ret = virgl_encoder_write_constant_buffer(&ctx, PIPE_SHADER_FRAGMENT, 2, 0, NULL);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+   ctx.cbuf->cdw = 0;
+
+   /* Sending constant for non-existing shader type */
+   ret = virgl_encoder_write_constant_buffer(&ctx, PIPE_SHADER_INVALID, 4, 10, constant_buffer);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_decode_bind_sampler_states)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   uint32_t handles[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
+
+   /* Here we are using correct values, no error expected */
+   ret = virgl_encode_bind_sampler_states(&ctx, PIPE_SHADER_VERTEX, 0, 10, handles);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Sending constant for non-existing shader type */
+   ret = virgl_encode_bind_sampler_states(&ctx, PIPE_SHADER_INVALID, 0, 10, handles);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_test_encode_sampler_view)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_sampler_view state = {
+       .format = VIRGL_FORMAT_B8G8R8A8_UNORM
+   };
+
+   struct virgl_resource res;
+   struct virgl_resource res2;
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res2, 2, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* Here we are using correct values, no error expected */
+   ret = virgl_encode_sampler_view(&ctx, res.handle,
+                                   &res,
+                                   &state);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Sending illegal format */
+   state.format = VIRGL_FORMAT_NONE;
+   ret = virgl_encode_sampler_view(&ctx, res2.handle,
+                                   &res2,
+                                   &state);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Sending illegal format */
+   state.format = VIRGL_FORMAT_MAX;
+   ret = virgl_encode_sampler_view(&ctx, res2.handle,
+                                   &res2,
+                                   &state);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Sending illegal sampler view target */
+   state.format = VIRGL_FORMAT_B8G8R8A8_UNORM | PIPE_MAX_TEXTURE_TYPES << 24;
+   ret = virgl_encode_sampler_view(&ctx, res2.handle,
+                                   &res2,
+                                   &state);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+static void test_create_surface(enum pipe_format format, int expected_error)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res;
+   struct virgl_surface surf;
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* create a surface for the resource */
+   memset(&surf, 0, sizeof(surf));
+   surf.base.format = format;
+   surf.handle = 1;
+   surf.base.texture = &res.base;
+
+   ret = virgl_encoder_create_surface(&ctx, surf.handle, &res, &surf.base);
+   ck_assert_int_eq(ret, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, expected_error);
+
+   /* cleanup */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_test_create_surface_pass)
+{
+   test_create_surface(VIRGL_FORMAT_B8G8R8A8_UNORM, 0);
+}
+END_TEST
+
+START_TEST(virgl_test_create_surface_fail_format)
+{
+   test_create_surface(VIRGL_FORMAT_MAX, EINVAL);
+}
+END_TEST
+
 static Suite *virgl_init_suite(void)
 {
   Suite *s;
@@ -1043,6 +1322,13 @@ static Suite *virgl_init_suite(void)
   tcase_add_test(tc_core, virgl_test_render_simple);
   tcase_add_test(tc_core, virgl_test_render_geom_simple);
   tcase_add_test(tc_core, virgl_test_render_xfb);
+  tcase_add_test(tc_core, virgl_test_set_viewport_state);
+  tcase_add_test(tc_core, virgl_decode_set_scissor_state);
+  tcase_add_test(tc_core, virgl_decode_set_constant_buffer);
+  tcase_add_test(tc_core, virgl_decode_bind_sampler_states);
+  tcase_add_test(tc_core, virgl_test_encode_sampler_view);
+  tcase_add_test(tc_core, virgl_test_create_surface_pass);
+  tcase_add_test(tc_core, virgl_test_create_surface_fail_format);
 
   suite_add_tcase(s, tc_core);
   return s;
