@@ -25,11 +25,11 @@
 /* transfer and iov related tests */
 #include <check.h>
 #include <stdlib.h>
-#include <sys/uio.h>
 #include <errno.h>
 #include <virglrenderer.h>
 #include "pipe/p_defines.h"
 #include "virgl_hw.h"
+#include "vrend_iov.h"
 #include "virgl_protocol.h"
 #include "testvirgl_encode.h"
 
@@ -618,7 +618,7 @@ static unsigned get_box_size(struct pipe_box *box, int elsize)
 }
 
 static void virgl_test_transfer_res(enum pipe_texture_target target,
-				    bool write, bool invalid)
+                                    bool write, bool invalid)
 {
   struct virgl_renderer_resource_create_args res;
   struct pipe_box box;
@@ -642,10 +642,10 @@ static void virgl_test_transfer_res(enum pipe_texture_target target,
 
   if (write)
     ret = virgl_renderer_transfer_write_iov(res.handle, 1, 0, 0, 0,
-					 (struct virgl_box *)&box, 0, iovs, niovs);
+                                            (struct virgl_box *)&box, 0, iovs, niovs);
   else
     ret = virgl_renderer_transfer_read_iov(res.handle, 1, 0, 0, 0,
-					   (struct virgl_box *)&box, 0, iovs, niovs);
+                                           (struct virgl_box *)&box, 0, iovs, niovs);
   ck_assert_int_eq(ret, invalid ? EINVAL : 0);
   virgl_renderer_ctx_detach_resource(1, res.handle);
 
@@ -678,7 +678,7 @@ START_TEST(virgl_test_transfer_res_write_invalid)
 END_TEST
 
 static void virgl_test_transfer_inline(enum pipe_texture_target target,
-				       bool invalid, int large_flags)
+                                       bool invalid, int large_flags)
 {
   struct virgl_renderer_resource_create_args args;
   struct pipe_box box;
@@ -704,7 +704,7 @@ static void virgl_test_transfer_inline(enum pipe_texture_target target,
 
   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
   virgl_encoder_inline_write(&ctx, &res, 0, 0, (struct pipe_box *)&box, data, box.width * elsize, 0);
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, invalid ? EINVAL : 0);
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
 
@@ -749,7 +749,7 @@ START_TEST(virgl_test_transfer_to_staging_without_iov_fails)
   box.width = bufsize;
   virgl_encoder_transfer(&ctx, &res, 0, 0, &box, 0, VIRGL_TRANSFER_TO_HOST);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, EINVAL);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
@@ -776,7 +776,7 @@ START_TEST(virgl_test_transfer_to_staging_with_iov_succeeds)
   box.width = bufsize;
   virgl_encoder_transfer(&ctx, &res, 0, 0, &box, 0, VIRGL_TRANSFER_TO_HOST);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, 0);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
@@ -809,7 +809,7 @@ START_TEST(virgl_test_copy_transfer_from_staging_without_iov_fails)
   box.width = bufsize;
   virgl_encoder_copy_transfer(&ctx, &dst_res, 0, 0, &box, &src_res, 0, synchronized);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, EINVAL);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, src_res.handle);
@@ -844,7 +844,7 @@ START_TEST(virgl_test_copy_transfer_from_staging_with_iov_succeeds)
   box.width = bufsize;
   virgl_encoder_copy_transfer(&ctx, &dst_res, 0, 0, &box, &src_res, 0, synchronized);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, 0);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, src_res.handle);
@@ -878,7 +878,7 @@ START_TEST(virgl_test_copy_transfer_to_staging_without_iov_fails)
 
   virgl_encoder_copy_transfer(&ctx, &dst_res, 0, 0, &box, &src_res, 0, synchronized);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, EINVAL);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, src_res.handle);
@@ -912,7 +912,7 @@ START_TEST(virgl_test_copy_transfer_to_staging_with_iov_succeeds)
 
   virgl_encoder_copy_transfer(&ctx, &dst_res, 0, 0, &box, &src_res, 0, synchronized);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, 0);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, src_res.handle);
@@ -943,7 +943,7 @@ START_TEST(virgl_test_transfer_near_res_bounds_with_stride_succeeds)
   virgl_encoder_transfer_with_stride(&ctx, &res, 0, 0, &box, 6 * 4, VIRGL_TRANSFER_TO_HOST,
                                      res_stride, 0);
 
-  ret = virgl_renderer_submit_cmd(ctx.cbuf->buf, ctx.ctx_id, ctx.cbuf->cdw);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
   ck_assert_int_eq(ret, 0);
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
@@ -997,7 +997,7 @@ START_TEST(test_vrend_host_backed_memory_no_data_leak)
 
    virgl_renderer_resource_unref(res.handle);
    free(data);
-
+   testvirgl_fini_ctx_cmdbuf(&ctx);
 }
 END_TEST
 
@@ -1031,7 +1031,6 @@ static Suite *virgl_init_suite(void)
   tcase_add_test(tc_core, virgl_test_transfer_buffer_bad_strides);
   tcase_add_test(tc_core, virgl_test_transfer_2d_array_bad_layer_stride);
   tcase_add_test(tc_core, virgl_test_transfer_2d_bad_level);
-  tcase_add_test(tc_core, test_vrend_host_backed_memory_no_data_leak);
 
   tcase_add_loop_test(tc_core, virgl_test_transfer_res_read_valid, 0, PIPE_MAX_TEXTURE_TYPES);
   tcase_add_loop_test(tc_core, virgl_test_transfer_res_write_valid, 0, PIPE_MAX_TEXTURE_TYPES);
@@ -1039,6 +1038,10 @@ static Suite *virgl_init_suite(void)
   tcase_add_loop_test(tc_core, virgl_test_transfer_res_write_invalid, 0, PIPE_MAX_TEXTURE_TYPES);
   suite_add_tcase(s, tc_core);
 
+  tc_core = tcase_create("leak");
+  tcase_add_test(tc_core, test_vrend_host_backed_memory_no_data_leak);
+  suite_add_tcase(s, tc_core);
+
   tc_core = tcase_create("transfer_inline_write");
   tcase_add_loop_test(tc_core, virgl_test_transfer_inline_valid, 0, PIPE_MAX_TEXTURE_TYPES);
   tcase_add_loop_test(tc_core, virgl_test_transfer_inline_invalid, 0, PIPE_MAX_TEXTURE_TYPES);
