@@ -23,9 +23,10 @@
  **************************************************************************/
 
 /* helper functions for testing purposes */
+#include "config.h"
+
 #include <check.h>
 #include <errno.h>
-#include <sys/uio.h>
 #include "pipe/p_defines.h"
 #include "pipe/p_format.h"
 #include "util/u_memory.h"
@@ -33,8 +34,11 @@
 #include "testvirgl.h"
 
 #include "virgl_hw.h"
+#include "vrend_iov.h"
 #include "virglrenderer.h"
 
+int context_flags = VIRGL_RENDERER_USE_EGL;
+
 void testvirgl_init_simple_1d_resource(struct virgl_renderer_resource_create_args *res, int handle)
 {
     res->handle = handle;
@@ -116,10 +120,10 @@ int testvirgl_init_single_ctx(void)
 
     test_cbs.version = 1;
     test_cbs.write_fence = testvirgl_write_fence;
-    ret = virgl_renderer_init(&mystruct, VIRGL_RENDERER_USE_EGL, &test_cbs);
+    ret = virgl_renderer_init(&mystruct, context_flags, &test_cbs);
     ck_assert_int_eq(ret, 0);
     if (ret)
-	return ret;
+        return ret;
     ret = virgl_renderer_context_create(1, strlen("test1"), "test1");
     ck_assert_int_eq(ret, 0);
     return ret;
@@ -148,21 +152,21 @@ int testvirgl_init_ctx_cmdbuf(struct virgl_context *ctx)
     int ret;
     ret = testvirgl_init_single_ctx();
     if (ret)
-	return ret;
+        return ret;
 
     ctx->flush = testvirgl_flush;
     ctx->ctx_id = 1;
     ctx->cbuf = CALLOC_STRUCT(virgl_cmd_buf);
     if (!ctx->cbuf) {
-	testvirgl_fini_single_ctx();
-	return ENOMEM;
+        testvirgl_fini_single_ctx();
+        return ENOMEM;
     }
 
     ctx->cbuf->buf = CALLOC(1, VIRGL_MAX_CMDBUF_DWORDS * 4);
     if (!ctx->cbuf->buf) {
-	FREE(ctx->cbuf);
-	testvirgl_fini_single_ctx();
-	return ENOMEM;
+        FREE(ctx->cbuf);
+        testvirgl_fini_single_ctx();
+        return ENOMEM;
     }
     return 0;
 }
@@ -174,8 +178,15 @@ void testvirgl_fini_ctx_cmdbuf(struct virgl_context *ctx)
     testvirgl_fini_single_ctx();
 }
 
+int testvirgl_ctx_send_cmdbuf(struct virgl_context *ctx)
+{
+   int res = virgl_renderer_submit_cmd(ctx->cbuf->buf, ctx->ctx_id, ctx->cbuf->cdw);
+   ctx->cbuf->cdw = 0;
+   return res;
+}
+
 int testvirgl_create_backed_simple_2d_res(struct virgl_resource *res,
-					  int handle, int w, int h)
+                                          int handle, int w, int h)
 {
     struct virgl_renderer_resource_create_args args;
     uint32_t backing_size;
@@ -203,7 +214,7 @@ int testvirgl_create_backed_simple_2d_res(struct virgl_resource *res,
 }
 
 int testvirgl_create_backed_simple_1d_res(struct virgl_resource *res,
-					  int handle)
+                                          int handle)
 {
     struct virgl_renderer_resource_create_args args;
     uint32_t backing_size;
@@ -241,7 +252,7 @@ void testvirgl_destroy_backed_res(struct virgl_resource *res)
 }
 
 int testvirgl_create_backed_simple_buffer(struct virgl_resource *res,
-					  int handle, int size, int binding)
+                                          int handle, int size, int binding)
 {
     struct virgl_renderer_resource_create_args args;
     uint32_t backing_size;
@@ -267,6 +278,25 @@ int testvirgl_create_backed_simple_buffer(struct virgl_resource *res,
     return 0;
 }
 
+int testvirgl_create_unbacked_simple_buffer(struct virgl_resource *res,
+                                            int handle, int size, int binding)
+{
+    struct virgl_renderer_resource_create_args args;
+    int ret;
+
+    testvirgl_init_simple_buffer_sized(&args, handle, size);
+    args.bind = binding;
+    ret = virgl_renderer_resource_create(&args, NULL, 0);
+    ck_assert_int_eq(ret, 0);
+
+    res->handle = handle;
+    res->base.target = args.target;
+    res->base.format = args.format;
+    res->base.bind = args.bind;
+
+    return 0;
+}
+
 static void *get_caps(void)
 {
     uint32_t max_ver, max_size;
